
''' Generated by OTF2 Template Engine '''

import ctypes

from .Config import EnumBase, EnumBitset

class SystemTreeDomain(EnumBase, ctypes.c_uint8):
    pass

SYSTEM_TREE_DOMAIN_MACHINE = SystemTreeDomain._construct(0, "MACHINE")
SYSTEM_TREE_DOMAIN_SHARED_MEMORY = SystemTreeDomain._construct(1, "SHARED_MEMORY")
SYSTEM_TREE_DOMAIN_NUMA = SystemTreeDomain._construct(2, "NUMA")
SYSTEM_TREE_DOMAIN_SOCKET = SystemTreeDomain._construct(3, "SOCKET")
SYSTEM_TREE_DOMAIN_CACHE = SystemTreeDomain._construct(4, "CACHE")
SYSTEM_TREE_DOMAIN_CORE = SystemTreeDomain._construct(5, "CORE")
SYSTEM_TREE_DOMAIN_PU = SystemTreeDomain._construct(6, "PU")
SYSTEM_TREE_DOMAIN_ACCELERATOR_DEVICE = SystemTreeDomain._construct(7, "ACCELERATOR_DEVICE")
SYSTEM_TREE_DOMAIN_NETWORKING_DEVICE = SystemTreeDomain._construct(8, "NETWORKING_DEVICE")

class LocationGroupType(EnumBase, ctypes.c_uint8):
    pass

LOCATION_GROUP_TYPE_UNKNOWN = LocationGroupType._construct(0, "UNKNOWN")
LOCATION_GROUP_TYPE_PROCESS = LocationGroupType._construct(1, "PROCESS")
LOCATION_GROUP_TYPE_ACCELERATOR = LocationGroupType._construct(2, "ACCELERATOR")

class LocationType(EnumBase, ctypes.c_uint8):
    pass

LOCATION_TYPE_UNKNOWN = LocationType._construct(0, "UNKNOWN")
LOCATION_TYPE_CPU_THREAD = LocationType._construct(1, "CPU_THREAD")
LOCATION_TYPE_ACCELERATOR_STREAM = LocationType._construct(2, "ACCELERATOR_STREAM")
LOCATION_TYPE_GPU = LocationType._construct(2, "GPU")
LOCATION_TYPE_METRIC = LocationType._construct(3, "METRIC")

class RegionRole(EnumBase, ctypes.c_uint8):
    pass

REGION_ROLE_UNKNOWN = RegionRole._construct(0, "UNKNOWN")
REGION_ROLE_FUNCTION = RegionRole._construct(1, "FUNCTION")
REGION_ROLE_WRAPPER = RegionRole._construct(2, "WRAPPER")
REGION_ROLE_LOOP = RegionRole._construct(3, "LOOP")
REGION_ROLE_CODE = RegionRole._construct(4, "CODE")
REGION_ROLE_PARALLEL = RegionRole._construct(5, "PARALLEL")
REGION_ROLE_SECTIONS = RegionRole._construct(6, "SECTIONS")
REGION_ROLE_SECTION = RegionRole._construct(7, "SECTION")
REGION_ROLE_WORKSHARE = RegionRole._construct(8, "WORKSHARE")
REGION_ROLE_SINGLE = RegionRole._construct(9, "SINGLE")
REGION_ROLE_SINGLE_SBLOCK = RegionRole._construct(10, "SINGLE_SBLOCK")
REGION_ROLE_MASTER = RegionRole._construct(11, "MASTER")
REGION_ROLE_CRITICAL = RegionRole._construct(12, "CRITICAL")
REGION_ROLE_CRITICAL_SBLOCK = RegionRole._construct(13, "CRITICAL_SBLOCK")
REGION_ROLE_ATOMIC = RegionRole._construct(14, "ATOMIC")
REGION_ROLE_BARRIER = RegionRole._construct(15, "BARRIER")
REGION_ROLE_IMPLICIT_BARRIER = RegionRole._construct(16, "IMPLICIT_BARRIER")
REGION_ROLE_FLUSH = RegionRole._construct(17, "FLUSH")
REGION_ROLE_ORDERED = RegionRole._construct(18, "ORDERED")
REGION_ROLE_ORDERED_SBLOCK = RegionRole._construct(19, "ORDERED_SBLOCK")
REGION_ROLE_TASK = RegionRole._construct(20, "TASK")
REGION_ROLE_TASK_CREATE = RegionRole._construct(21, "TASK_CREATE")
REGION_ROLE_TASK_WAIT = RegionRole._construct(22, "TASK_WAIT")
REGION_ROLE_COLL_ONE2ALL = RegionRole._construct(23, "COLL_ONE2ALL")
REGION_ROLE_COLL_ALL2ONE = RegionRole._construct(24, "COLL_ALL2ONE")
REGION_ROLE_COLL_ALL2ALL = RegionRole._construct(25, "COLL_ALL2ALL")
REGION_ROLE_COLL_OTHER = RegionRole._construct(26, "COLL_OTHER")
REGION_ROLE_FILE_IO = RegionRole._construct(27, "FILE_IO")
REGION_ROLE_POINT2POINT = RegionRole._construct(28, "POINT2POINT")
REGION_ROLE_RMA = RegionRole._construct(29, "RMA")
REGION_ROLE_DATA_TRANSFER = RegionRole._construct(30, "DATA_TRANSFER")
REGION_ROLE_ARTIFICIAL = RegionRole._construct(31, "ARTIFICIAL")
REGION_ROLE_THREAD_CREATE = RegionRole._construct(32, "THREAD_CREATE")
REGION_ROLE_THREAD_WAIT = RegionRole._construct(33, "THREAD_WAIT")
REGION_ROLE_TASK_UNTIED = RegionRole._construct(34, "TASK_UNTIED")
REGION_ROLE_ALLOCATE = RegionRole._construct(35, "ALLOCATE")
REGION_ROLE_DEALLOCATE = RegionRole._construct(36, "DEALLOCATE")
REGION_ROLE_REALLOCATE = RegionRole._construct(37, "REALLOCATE")
REGION_ROLE_FILE_IO_METADATA = RegionRole._construct(38, "FILE_IO_METADATA")
REGION_ROLE_CANCEL = RegionRole._construct(39, "CANCEL")
REGION_ROLE_KERNEL = RegionRole._construct(40, "KERNEL")

class RegionFlag(EnumBitset, ctypes.c_uint32):
    pass

REGION_FLAG_NONE = RegionFlag._construct(0, "NONE")
REGION_FLAG_DYNAMIC = RegionFlag._construct(( 1 << 0 ), "DYNAMIC")
REGION_FLAG_PHASE = RegionFlag._construct(( 1 << 1 ), "PHASE")

class GroupType(EnumBase, ctypes.c_uint8):
    pass

GROUP_TYPE_UNKNOWN = GroupType._construct(0, "UNKNOWN")
GROUP_TYPE_LOCATIONS = GroupType._construct(1, "LOCATIONS")
GROUP_TYPE_REGIONS = GroupType._construct(2, "REGIONS")
GROUP_TYPE_METRIC = GroupType._construct(3, "METRIC")
GROUP_TYPE_COMM_LOCATIONS = GroupType._construct(4, "COMM_LOCATIONS")
GROUP_TYPE_COMM_GROUP = GroupType._construct(5, "COMM_GROUP")
GROUP_TYPE_COMM_SELF = GroupType._construct(6, "COMM_SELF")

class GroupFlag(EnumBitset, ctypes.c_uint32):
    pass

GROUP_FLAG_NONE = GroupFlag._construct(0, "NONE")
GROUP_FLAG_GLOBAL_MEMBERS = GroupFlag._construct(( 1 << 0 ), "GLOBAL_MEMBERS")

class Base(EnumBase, ctypes.c_uint8):
    pass

BASE_BINARY = Base._construct(0, "BINARY")
BASE_DECIMAL = Base._construct(1, "DECIMAL")

class MetricOccurrence(EnumBase, ctypes.c_uint8):
    pass

METRIC_SYNCHRONOUS_STRICT = MetricOccurrence._construct(0, "SYNCHRONOUS_STRICT")
METRIC_SYNCHRONOUS = MetricOccurrence._construct(1, "SYNCHRONOUS")
METRIC_ASYNCHRONOUS = MetricOccurrence._construct(2, "ASYNCHRONOUS")

class MetricType(EnumBase, ctypes.c_uint8):
    pass

METRIC_TYPE_OTHER = MetricType._construct(0, "OTHER")
METRIC_TYPE_PAPI = MetricType._construct(1, "PAPI")
METRIC_TYPE_RUSAGE = MetricType._construct(2, "RUSAGE")
METRIC_TYPE_USER = MetricType._construct(3, "USER")

class MetricValueProperty(EnumBase, ctypes.c_uint8):
    pass

METRIC_VALUE_ACCUMULATED = MetricValueProperty._construct(0, "ACCUMULATED")
METRIC_VALUE_ABSOLUTE = MetricValueProperty._construct(1, "ABSOLUTE")
METRIC_VALUE_RELATIVE = MetricValueProperty._construct(2, "RELATIVE")
METRIC_VALUE_MASK = MetricValueProperty(15)

class MetricTiming(EnumBase, ctypes.c_uint8):
    pass

METRIC_TIMING_START = MetricTiming._construct(0, "START")
METRIC_TIMING_POINT = MetricTiming._construct(1 << 4, "POINT")
METRIC_TIMING_LAST = MetricTiming._construct(2 << 4, "LAST")
METRIC_TIMING_NEXT = MetricTiming._construct(3 << 4, "NEXT")
METRIC_TIMING_MASK = MetricTiming(240)

class MetricMode(EnumBase, ctypes.c_uint8):
    pass


class MetricScope(EnumBase, ctypes.c_uint8):
    pass

SCOPE_LOCATION = MetricScope._construct(0, "LOCATION")
SCOPE_LOCATION_GROUP = MetricScope._construct(1, "LOCATION_GROUP")
SCOPE_SYSTEM_TREE_NODE = MetricScope._construct(2, "SYSTEM_TREE_NODE")
SCOPE_GROUP = MetricScope._construct(3, "GROUP")

class RecorderKind(EnumBase, ctypes.c_uint8):
    pass

RECORDER_KIND_UNKNOWN = RecorderKind._construct(0, "UNKNOWN")
RECORDER_KIND_ABSTRACT = RecorderKind._construct(1, "ABSTRACT")
RECORDER_KIND_CPU = RecorderKind._construct(2, "CPU")
RECORDER_KIND_GPU = RecorderKind._construct(3, "GPU")

class ParameterType(EnumBase, ctypes.c_uint8):
    pass

PARAMETER_TYPE_STRING = ParameterType._construct(0, "STRING")
PARAMETER_TYPE_INT64 = ParameterType._construct(1, "INT64")
PARAMETER_TYPE_UINT64 = ParameterType._construct(2, "UINT64")

class CartPeriodicity(EnumBase, ctypes.c_uint8):
    pass

CART_PERIODIC_FALSE = CartPeriodicity._construct(0, "FALSE")
CART_PERIODIC_TRUE = CartPeriodicity._construct(1, "TRUE")

class InterruptGeneratorMode(EnumBase, ctypes.c_uint8):
    pass

INTERRUPT_GENERATOR_MODE_TIME = InterruptGeneratorMode._construct(0, "TIME")
INTERRUPT_GENERATOR_MODE_COUNT = InterruptGeneratorMode._construct(1, "COUNT")

class IoParadigmClass(EnumBase, ctypes.c_uint8):
    pass

IO_PARADIGM_CLASS_SERIAL = IoParadigmClass._construct(0, "SERIAL")
IO_PARADIGM_CLASS_PARALLEL = IoParadigmClass._construct(1, "PARALLEL")

class IoParadigmFlag(EnumBitset, ctypes.c_uint32):
    pass

IO_PARADIGM_FLAG_NONE = IoParadigmFlag._construct(0, "NONE")
IO_PARADIGM_FLAG_OS = IoParadigmFlag._construct((1 << 0), "OS")

class IoParadigmProperty(EnumBase, ctypes.c_uint8):
    pass

IO_PARADIGM_PROPERTY_VERSION = IoParadigmProperty._construct(0, "VERSION")

class IoHandleFlag(EnumBitset, ctypes.c_uint32):
    pass

IO_HANDLE_FLAG_NONE = IoHandleFlag._construct(0, "NONE")
IO_HANDLE_FLAG_PRE_CREATED = IoHandleFlag._construct(( 1 << 0 ), "PRE_CREATED")
IO_HANDLE_FLAG_ALL_PROXY = IoHandleFlag._construct(( 1 << 1 ), "ALL_PROXY")

class IoAccessMode(EnumBase, ctypes.c_uint8):
    pass

IO_ACCESS_MODE_READ_ONLY = IoAccessMode._construct(0, "READ_ONLY")
IO_ACCESS_MODE_WRITE_ONLY = IoAccessMode._construct(1, "WRITE_ONLY")
IO_ACCESS_MODE_READ_WRITE = IoAccessMode._construct(2, "READ_WRITE")
IO_ACCESS_MODE_EXECUTE_ONLY = IoAccessMode._construct(3, "EXECUTE_ONLY")
IO_ACCESS_MODE_SEARCH_ONLY = IoAccessMode._construct(4, "SEARCH_ONLY")

class IoStatusFlag(EnumBitset, ctypes.c_uint32):
    pass

IO_STATUS_FLAG_NONE = IoStatusFlag._construct(0, "NONE")
IO_STATUS_FLAG_CLOSE_ON_EXEC = IoStatusFlag._construct(1 << 0, "CLOSE_ON_EXEC")
IO_STATUS_FLAG_APPEND = IoStatusFlag._construct(1 << 1, "APPEND")
IO_STATUS_FLAG_NON_BLOCKING = IoStatusFlag._construct(1 << 2, "NON_BLOCKING")
IO_STATUS_FLAG_ASYNC = IoStatusFlag._construct(1 << 3, "ASYNC")
IO_STATUS_FLAG_SYNC = IoStatusFlag._construct(1 << 4, "SYNC")
IO_STATUS_FLAG_DATA_SYNC = IoStatusFlag._construct(1 << 5, "DATA_SYNC")
IO_STATUS_FLAG_AVOID_CACHING = IoStatusFlag._construct(1 << 6, "AVOID_CACHING")
IO_STATUS_FLAG_NO_ACCESS_TIME = IoStatusFlag._construct(1 << 7, "NO_ACCESS_TIME")
IO_STATUS_FLAG_DELETE_ON_CLOSE = IoStatusFlag._construct(1 << 8, "DELETE_ON_CLOSE")

class CommFlag(EnumBitset, ctypes.c_uint32):
    pass

COMM_FLAG_NONE = CommFlag._construct(0, "NONE")
COMM_FLAG_CREATE_DESTROY_EVENTS = CommFlag._construct(1 << 0, "CREATE_DESTROY_EVENTS")

class RmaWinFlag(EnumBitset, ctypes.c_uint32):
    pass

RMA_WIN_FLAG_NONE = RmaWinFlag._construct(0, "NONE")
RMA_WIN_FLAG_CREATE_DESTROY_EVENTS = RmaWinFlag._construct(1 << 0, "CREATE_DESTROY_EVENTS")

METRIC_ACCUMULATED_START = MetricMode._construct(METRIC_VALUE_ACCUMULATED.value | METRIC_TIMING_START.value, "ACCUMULATED_START")
METRIC_ACCUMULATED_POINT = MetricMode._construct(METRIC_VALUE_ACCUMULATED.value | METRIC_TIMING_POINT.value, "ACCUMULATED_POINT")
METRIC_ACCUMULATED_LAST  = MetricMode._construct(METRIC_VALUE_ACCUMULATED.value | METRIC_TIMING_LAST.value, "ACCUMULATED_LAST")
METRIC_ACCUMULATED_NEXT  = MetricMode._construct(METRIC_VALUE_ACCUMULATED.value | METRIC_TIMING_NEXT.value, "ACCUMULATED_NEXT")
METRIC_ABSOLUTE_POINT    = MetricMode._construct(METRIC_VALUE_ABSOLUTE.value    | METRIC_TIMING_POINT.value, "ABSOLUTE_POINT")
METRIC_ABSOLUTE_LAST     = MetricMode._construct(METRIC_VALUE_ABSOLUTE.value    | METRIC_TIMING_LAST.value, "ABSOLUTE_LAST")
METRIC_ABSOLUTE_NEXT     = MetricMode._construct(METRIC_VALUE_ABSOLUTE.value    | METRIC_TIMING_NEXT.value, "ABSOLUTE_NEXT")
METRIC_RELATIVE_POINT    = MetricMode._construct(METRIC_VALUE_RELATIVE.value    | METRIC_TIMING_POINT.value, "RELATIVE_POINT")
METRIC_RELATIVE_LAST     = MetricMode._construct(METRIC_VALUE_RELATIVE.value    | METRIC_TIMING_LAST.value, "RELATIVE_LAST")
METRIC_RELATIVE_NEXT     = MetricMode._construct(METRIC_VALUE_RELATIVE.value    | METRIC_TIMING_NEXT.value, "RELATIVE_NEXT")

__all__ = [
    'SystemTreeDomain',
    'SYSTEM_TREE_DOMAIN_MACHINE',
    'SYSTEM_TREE_DOMAIN_SHARED_MEMORY',
    'SYSTEM_TREE_DOMAIN_NUMA',
    'SYSTEM_TREE_DOMAIN_SOCKET',
    'SYSTEM_TREE_DOMAIN_CACHE',
    'SYSTEM_TREE_DOMAIN_CORE',
    'SYSTEM_TREE_DOMAIN_PU',
    'SYSTEM_TREE_DOMAIN_ACCELERATOR_DEVICE',
    'SYSTEM_TREE_DOMAIN_NETWORKING_DEVICE',
    'LocationGroupType',
    'LOCATION_GROUP_TYPE_UNKNOWN',
    'LOCATION_GROUP_TYPE_PROCESS',
    'LOCATION_GROUP_TYPE_ACCELERATOR',
    'LocationType',
    'LOCATION_TYPE_UNKNOWN',
    'LOCATION_TYPE_CPU_THREAD',
    'LOCATION_TYPE_ACCELERATOR_STREAM',
    'LOCATION_TYPE_METRIC',
    'RegionRole',
    'REGION_ROLE_UNKNOWN',
    'REGION_ROLE_FUNCTION',
    'REGION_ROLE_WRAPPER',
    'REGION_ROLE_LOOP',
    'REGION_ROLE_CODE',
    'REGION_ROLE_PARALLEL',
    'REGION_ROLE_SECTIONS',
    'REGION_ROLE_SECTION',
    'REGION_ROLE_WORKSHARE',
    'REGION_ROLE_SINGLE',
    'REGION_ROLE_SINGLE_SBLOCK',
    'REGION_ROLE_MASTER',
    'REGION_ROLE_CRITICAL',
    'REGION_ROLE_CRITICAL_SBLOCK',
    'REGION_ROLE_ATOMIC',
    'REGION_ROLE_BARRIER',
    'REGION_ROLE_IMPLICIT_BARRIER',
    'REGION_ROLE_FLUSH',
    'REGION_ROLE_ORDERED',
    'REGION_ROLE_ORDERED_SBLOCK',
    'REGION_ROLE_TASK',
    'REGION_ROLE_TASK_CREATE',
    'REGION_ROLE_TASK_WAIT',
    'REGION_ROLE_COLL_ONE2ALL',
    'REGION_ROLE_COLL_ALL2ONE',
    'REGION_ROLE_COLL_ALL2ALL',
    'REGION_ROLE_COLL_OTHER',
    'REGION_ROLE_FILE_IO',
    'REGION_ROLE_POINT2POINT',
    'REGION_ROLE_RMA',
    'REGION_ROLE_DATA_TRANSFER',
    'REGION_ROLE_ARTIFICIAL',
    'REGION_ROLE_THREAD_CREATE',
    'REGION_ROLE_THREAD_WAIT',
    'REGION_ROLE_TASK_UNTIED',
    'REGION_ROLE_ALLOCATE',
    'REGION_ROLE_DEALLOCATE',
    'REGION_ROLE_REALLOCATE',
    'REGION_ROLE_FILE_IO_METADATA',
    'REGION_ROLE_CANCEL',
    'REGION_ROLE_KERNEL',
    'RegionFlag',
    'REGION_FLAG_NONE',
    'REGION_FLAG_DYNAMIC',
    'REGION_FLAG_PHASE',
    'GroupType',
    'GROUP_TYPE_UNKNOWN',
    'GROUP_TYPE_LOCATIONS',
    'GROUP_TYPE_REGIONS',
    'GROUP_TYPE_METRIC',
    'GROUP_TYPE_COMM_LOCATIONS',
    'GROUP_TYPE_COMM_GROUP',
    'GROUP_TYPE_COMM_SELF',
    'GroupFlag',
    'GROUP_FLAG_NONE',
    'GROUP_FLAG_GLOBAL_MEMBERS',
    'Base',
    'BASE_BINARY',
    'BASE_DECIMAL',
    'MetricOccurrence',
    'METRIC_SYNCHRONOUS_STRICT',
    'METRIC_SYNCHRONOUS',
    'METRIC_ASYNCHRONOUS',
    'MetricType',
    'METRIC_TYPE_OTHER',
    'METRIC_TYPE_PAPI',
    'METRIC_TYPE_RUSAGE',
    'METRIC_TYPE_USER',
    'MetricValueProperty',
    'METRIC_VALUE_ACCUMULATED',
    'METRIC_VALUE_ABSOLUTE',
    'METRIC_VALUE_RELATIVE',
    'METRIC_VALUE_MASK',
    'MetricTiming',
    'METRIC_TIMING_START',
    'METRIC_TIMING_POINT',
    'METRIC_TIMING_LAST',
    'METRIC_TIMING_NEXT',
    'METRIC_TIMING_MASK',
    'MetricMode',
    'METRIC_ACCUMULATED_START',
    'METRIC_ACCUMULATED_POINT',
    'METRIC_ACCUMULATED_LAST',
    'METRIC_ACCUMULATED_NEXT',
    'METRIC_ABSOLUTE_POINT',
    'METRIC_ABSOLUTE_LAST',
    'METRIC_ABSOLUTE_NEXT',
    'METRIC_RELATIVE_POINT',
    'METRIC_RELATIVE_LAST',
    'METRIC_RELATIVE_NEXT',
    'MetricScope',
    'SCOPE_LOCATION',
    'SCOPE_LOCATION_GROUP',
    'SCOPE_SYSTEM_TREE_NODE',
    'SCOPE_GROUP',
    'RecorderKind',
    'RECORDER_KIND_UNKNOWN',
    'RECORDER_KIND_ABSTRACT',
    'RECORDER_KIND_CPU',
    'RECORDER_KIND_GPU',
    'ParameterType',
    'PARAMETER_TYPE_STRING',
    'PARAMETER_TYPE_INT64',
    'PARAMETER_TYPE_UINT64',
    'CartPeriodicity',
    'CART_PERIODIC_FALSE',
    'CART_PERIODIC_TRUE',
    'InterruptGeneratorMode',
    'INTERRUPT_GENERATOR_MODE_TIME',
    'INTERRUPT_GENERATOR_MODE_COUNT',
    'IoParadigmClass',
    'IO_PARADIGM_CLASS_SERIAL',
    'IO_PARADIGM_CLASS_PARALLEL',
    'IoParadigmFlag',
    'IO_PARADIGM_FLAG_NONE',
    'IO_PARADIGM_FLAG_OS',
    'IoParadigmProperty',
    'IO_PARADIGM_PROPERTY_VERSION',
    'IoHandleFlag',
    'IO_HANDLE_FLAG_NONE',
    'IO_HANDLE_FLAG_PRE_CREATED',
    'IO_HANDLE_FLAG_ALL_PROXY',
    'IoAccessMode',
    'IO_ACCESS_MODE_READ_ONLY',
    'IO_ACCESS_MODE_WRITE_ONLY',
    'IO_ACCESS_MODE_READ_WRITE',
    'IO_ACCESS_MODE_EXECUTE_ONLY',
    'IO_ACCESS_MODE_SEARCH_ONLY',
    'IoStatusFlag',
    'IO_STATUS_FLAG_NONE',
    'IO_STATUS_FLAG_CLOSE_ON_EXEC',
    'IO_STATUS_FLAG_APPEND',
    'IO_STATUS_FLAG_NON_BLOCKING',
    'IO_STATUS_FLAG_ASYNC',
    'IO_STATUS_FLAG_SYNC',
    'IO_STATUS_FLAG_DATA_SYNC',
    'IO_STATUS_FLAG_AVOID_CACHING',
    'IO_STATUS_FLAG_NO_ACCESS_TIME',
    'IO_STATUS_FLAG_DELETE_ON_CLOSE',
    'CommFlag',
    'COMM_FLAG_NONE',
    'COMM_FLAG_CREATE_DESTROY_EVENTS',
    'RmaWinFlag',
    'RMA_WIN_FLAG_NONE',
    'RMA_WIN_FLAG_CREATE_DESTROY_EVENTS',
]



import ctypes

from .Config import conf, StrParam, str_decode, free
from .ErrorCodes import ErrorCode, HandleErrorCode, errcheck_not_null
from .GeneralDefinitions import Compression, FileSubstrate, Hint, FileMode, LocationRef, ThumbnailType
from .Callbacks import FlushCallbacks, LockingCallbacks, MemoryCallbacks, CollectiveCallbacks, CollectiveContext
from .DefWriter import DefWriter
from .DefReader import DefReader
from .EvtWriter import EvtWriter
from .EvtReader import EvtReader
from .SnapWriter import SnapWriter
from .SnapReader import SnapReader
from .GlobalDefWriter import GlobalDefWriter
from .GlobalDefReader import GlobalDefReader
from .GlobalEvtReader import GlobalEvtReader
from .GlobalSnapReader import GlobalSnapReader
from .Thumbnail import ThumbWriter, ThumbReader
from .MarkerWriter import MarkerWriter
from .MarkerReader import MarkerReader


class Archive(ctypes.Structure):
    pass

def Archive_Open(archivePath, archiveName, fileMode, chunkSizeEvents, chunkSizeDefs, fileSubstrate, compression):
    c_Open = conf.lib.OTF2_Archive_Open
    c_Open.argtypes = [ StrParam, StrParam, FileMode, ctypes.c_uint64, ctypes.c_uint64, FileSubstrate, Compression ]
    c_Open.restype = ctypes.POINTER(Archive)
    c_Open.errcheck = errcheck_not_null
    return c_Open(archivePath, archiveName, fileMode, chunkSizeEvents, chunkSizeDefs, fileSubstrate, compression)

def Archive_Close(archive):
    c_Close = conf.lib.OTF2_Archive_Close
    c_Close.argtypes = [ ctypes.POINTER(Archive) ]
    c_Close.restype = ErrorCode
    c_Close.errcheck = HandleErrorCode
    c_Close(archive)

def Archive_SwitchFileMode(archive, newFileMode):
    c_SwitchFileMode = conf.lib.OTF2_Archive_SwitchFileMode
    c_SwitchFileMode.argtypes = [ ctypes.POINTER(Archive), FileMode ]
    c_SwitchFileMode.restype = ErrorCode
    c_SwitchFileMode.errcheck = HandleErrorCode
    c_SwitchFileMode(archive, newFileMode)

def Archive_SetDefChunkSize(archive, chunkSizeDefs):
    c_SetDefChunkSize = conf.lib.OTF2_Archive_SetDefChunkSize
    c_SetDefChunkSize.argtypes = [ ctypes.POINTER(Archive), ctypes.c_uint64 ]
    c_SetDefChunkSize.restype = ErrorCode
    c_SetDefChunkSize.errcheck = HandleErrorCode
    c_SetDefChunkSize(archive, chunkSizeDefs)

def Archive_SetMachineName(archive, machineName):
    c_SetMachineName = conf.lib.OTF2_Archive_SetMachineName
    c_SetMachineName.argtypes = [ ctypes.POINTER(Archive), StrParam ]
    c_SetMachineName.restype = ErrorCode
    c_SetMachineName.errcheck = HandleErrorCode
    c_SetMachineName(archive, machineName)

def Archive_SetDescription(archive, description):
    c_SetDescription = conf.lib.OTF2_Archive_SetDescription
    c_SetDescription.argtypes = [ ctypes.POINTER(Archive), StrParam ]
    c_SetDescription.restype = ErrorCode
    c_SetDescription.errcheck = HandleErrorCode
    c_SetDescription(archive, description)

def Archive_SetCreator(archive, creator):
    c_SetCreator = conf.lib.OTF2_Archive_SetCreator
    c_SetCreator.argtypes = [ ctypes.POINTER(Archive), StrParam ]
    c_SetCreator.restype = ErrorCode
    c_SetCreator.errcheck = HandleErrorCode
    c_SetCreator(archive, creator)

def Archive_SetFlushCallbacks(archive, flushCallbacks, flushData):
    c_SetFlushCallbacks = conf.lib.OTF2_Archive_SetFlushCallbacks
    c_SetFlushCallbacks.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(FlushCallbacks), ctypes.py_object ]
    c_SetFlushCallbacks.restype = ErrorCode
    c_SetFlushCallbacks.errcheck = HandleErrorCode
    c_SetFlushCallbacks(archive, ctypes.byref(flushCallbacks), ctypes.py_object(flushData))

def Archive_SetMemoryCallbacks(archive, memoryCallbacks, memoryData):
    c_SetMemoryCallbacks = conf.lib.OTF2_Archive_SetMemoryCallbacks
    c_SetMemoryCallbacks.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(MemoryCallbacks), ctypes.py_object ]
    c_SetMemoryCallbacks.restype = ErrorCode
    c_SetMemoryCallbacks.errcheck = HandleErrorCode
    c_SetMemoryCallbacks(archive, ctypes.byref(memoryCallbacks), ctypes.py_object(memoryData))

def Archive_SetCollectiveCallbacks(archive, collectiveCallbacks, collectiveData, globalCommContext, localCommContext):
    c_SetCollectiveCallbacks = conf.lib.OTF2_Archive_SetCollectiveCallbacks
    c_SetCollectiveCallbacks.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(CollectiveCallbacks), ctypes.py_object, ctypes.POINTER(CollectiveContext), ctypes.POINTER(CollectiveContext) ]
    c_SetCollectiveCallbacks.restype = ErrorCode
    c_SetCollectiveCallbacks.errcheck = HandleErrorCode
    c_SetCollectiveCallbacks(archive, ctypes.byref(collectiveCallbacks), ctypes.py_object(collectiveData), ctypes.byref(globalCommContext), ctypes.byref(localCommContext))

def Archive_SetSerialCollectiveCallbacks(archive):
    c_SetSerialCollectiveCallbacks = conf.lib.OTF2_Archive_SetSerialCollectiveCallbacks
    c_SetSerialCollectiveCallbacks.argtypes = [ ctypes.POINTER(Archive) ]
    c_SetSerialCollectiveCallbacks.restype = ErrorCode
    c_SetSerialCollectiveCallbacks.errcheck = HandleErrorCode
    c_SetSerialCollectiveCallbacks(archive)

def Archive_IsPrimary(archive):
    c_IsPrimary = conf.lib.OTF2_Archive_IsPrimary
    c_IsPrimary.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_bool) ]
    c_IsPrimary.restype = ErrorCode
    c_IsPrimary.errcheck = HandleErrorCode
    result = ctypes.c_bool()
    c_IsPrimary(archive, ctypes.byref(result))
    return result.value

def Archive_IsMaster(archive):
    return Archive_IsPrimary(archive)

def Archive_SetLockingCallbacks(archive, lockingCallbacks, lockingData):
    c_SetLockingCallbacks = conf.lib.OTF2_Archive_SetLockingCallbacks
    c_SetLockingCallbacks.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(LockingCallbacks), ctypes.py_object ]
    c_SetLockingCallbacks.restype = ErrorCode
    c_SetLockingCallbacks.errcheck = HandleErrorCode
    c_SetLockingCallbacks(archive, ctypes.byref(lockingCallbacks), ctypes.py_object(lockingData))

def Archive_SetHint(archive, hint, value):
    c_SetHint = conf.lib.OTF2_Archive_SetHint
    c_SetHint.argtypes = [ ctypes.POINTER(Archive), Hint, ctypes.c_void_p ]
    c_SetHint.restype = ErrorCode
    c_SetHint.errcheck = HandleErrorCode
    c_SetHint(archive, hint, ctypes.byref(value))

def Archive_SetProperty(archive, name, value, overwrite):
    c_SetProperty = conf.lib.OTF2_Archive_SetProperty
    c_SetProperty.argtypes = [ ctypes.POINTER(Archive), StrParam, StrParam, ctypes.c_bool ]
    c_SetProperty.restype = ErrorCode
    c_SetProperty.errcheck = HandleErrorCode
    c_SetProperty(archive, name, value, overwrite)

def Archive_SetBoolProperty(archive, name, value, overwrite):
    c_SetBoolProperty = conf.lib.OTF2_Archive_SetBoolProperty
    c_SetBoolProperty.argtypes = [ ctypes.POINTER(Archive), StrParam, ctypes.c_bool, ctypes.c_bool ]
    c_SetBoolProperty.restype = ErrorCode
    c_SetBoolProperty.errcheck = HandleErrorCode
    c_SetBoolProperty(archive, name, ctypes.c_bool(value), ctypes.c_bool(overwrite))

def Archive_GetPropertyNames(archive):
    c_GetPropertyNames = conf.lib.OTF2_Archive_GetPropertyNames
    c_GetPropertyNames.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_uint32), ctypes.POINTER(ctypes.POINTER(ctypes.c_char_p)) ]
    c_GetPropertyNames.restype = ErrorCode
    c_GetPropertyNames.errcheck = HandleErrorCode
    numberOfProperties = ctypes.c_uint32()
    names = ctypes.POINTER(ctypes.c_char_p)()

    c_GetPropertyNames(archive, ctypes.byref(numberOfProperties), ctypes.byref(names))

    result = [str_decode(names[i]) for i in range(numberOfProperties.value)]
    free(names)

    return result

def Archive_GetProperty(archive, name):
    c_GetProperty = conf.lib.OTF2_Archive_GetProperty
    c_GetProperty.argtypes = [ ctypes.POINTER(Archive), StrParam, ctypes.POINTER(ctypes.c_char_p) ]
    c_GetProperty.restype = ErrorCode
    c_GetProperty.errcheck = HandleErrorCode
    value = ctypes.c_char_p()

    c_GetProperty(archive, name, ctypes.byref(value))

    result = str_decode(value.value)
    free(value)

    return result

def Archive_GetTraceId(archive):
    c_GetTraceId = conf.lib.OTF2_Archive_GetTraceId
    c_GetTraceId.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_uint64) ]
    c_GetTraceId.restype = ErrorCode
    c_GetTraceId.errcheck = HandleErrorCode
    trace_id = ctypes.c_uint64()
    c_GetTraceId(archive, ctypes.byref(trace_id))
    return trace_id.value

def Archive_GetNumberOfLocations(archive):
    c_GetNumberOfLocations = conf.lib.OTF2_Archive_GetNumberOfLocations
    c_GetNumberOfLocations.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_uint64) ]
    c_GetNumberOfLocations.restype = ErrorCode
    c_GetNumberOfLocations.errcheck = HandleErrorCode
    numberOfLocations = ctypes.c_uint64()
    c_GetNumberOfLocations(archive, ctypes.byref(numberOfLocations))
    return numberOfLocations.value

def Archive_GetNumberOfGlobalDefinitions(archive):
    c_GetNumberOfGlobalDefinitions = conf.lib.OTF2_Archive_GetNumberOfGlobalDefinitions
    c_GetNumberOfGlobalDefinitions.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_uint64) ]
    c_GetNumberOfGlobalDefinitions.restype = ErrorCode
    c_GetNumberOfGlobalDefinitions.errcheck = HandleErrorCode
    numberOfDefinitions = ctypes.c_uint64()
    c_GetNumberOfGlobalDefinitions(archive, ctypes.byref(numberOfDefinitions))
    return numberOfDefinitions.value

def Archive_GetMachineName(archive):
    c_GetMachineName = conf.lib.OTF2_Archive_GetMachineName
    c_GetMachineName.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_char_p) ]
    c_GetMachineName.restype = ErrorCode
    c_GetMachineName.errcheck = HandleErrorCode
    machineName = ctypes.c_char_p()

    c_GetMachineName(archive, ctypes.byref(machineName))

    result = str_decode(machineName.value)
    free(machineName)

    return result

def Archive_GetDescription(archive):
    c_GetDescription = conf.lib.OTF2_Archive_GetDescription
    c_GetDescription.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_char_p) ]
    c_GetDescription.restype = ErrorCode
    c_GetDescription.errcheck = HandleErrorCode
    description = ctypes.c_char_p()

    c_GetDescription(archive, ctypes.byref(description))

    result = str_decode(description.value)
    free(description)

    return result

def Archive_GetCreator(archive):
    c_GetCreator = conf.lib.OTF2_Archive_GetCreator
    c_GetCreator.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_char_p) ]
    c_GetCreator.restype = ErrorCode
    c_GetCreator.errcheck = HandleErrorCode
    creator = ctypes.c_char_p()

    c_GetCreator(archive, ctypes.byref(creator))

    result = str_decode(creator.value)
    free(creator)

    return result

def Archive_GetVersion(archive):
    c_GetVersion = conf.lib.OTF2_Archive_GetVersion
    c_GetVersion.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_uint8), ctypes.POINTER(ctypes.c_uint8), ctypes.POINTER(ctypes.c_uint8) ]
    c_GetVersion.restype = ErrorCode
    c_GetVersion.errcheck = HandleErrorCode
    major = ctypes.c_uint8()
    minor = ctypes.c_uint8()
    bugfix = ctypes.c_uint8()
    c_GetVersion(archive, ctypes.byref(major), ctypes.byref(minor), ctypes.byref(bugfix))
    return (major.value, minor.value, bugfix.value)

def Archive_GetChunkSize(archive):
    c_GetChunkSize = conf.lib.OTF2_Archive_GetChunkSize
    c_GetChunkSize.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_uint64), ctypes.POINTER(ctypes.c_uint64) ]
    c_GetChunkSize.restype = ErrorCode
    c_GetChunkSize.errcheck = HandleErrorCode
    chunkSizeEvents = ctypes.c_uint64()
    chunkSizeDefs = ctypes.c_uint64()
    c_GetChunkSize(archive, ctypes.byref(chunkSizeEvents), ctypes.byref(chunkSizeDefs))
    return (chunkSizeEvents.value, chunkSizeDefs.value)

def Archive_GetFileSubstrate(archive):
    c_GetFileSubstrate = conf.lib.OTF2_Archive_GetFileSubstrate
    c_GetFileSubstrate.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(FileSubstrate) ]
    c_GetFileSubstrate.restype = ErrorCode
    c_GetFileSubstrate.errcheck = HandleErrorCode
    substrate = FileSubstrate()
    c_GetFileSubstrate(archive, ctypes.byref(substrate))
    return substrate

def Archive_GetCompression(archive):
    c_GetCompression = conf.lib.OTF2_Archive_GetCompression
    c_GetCompression.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(Compression) ]
    c_GetCompression.restype = ErrorCode
    c_GetCompression.errcheck = HandleErrorCode
    compression = Compression()
    c_GetCompression(archive, ctypes.byref(compression))
    return compression

def Archive_GetEvtWriter(archive, location):
    c_GetEvtWriter = conf.lib.OTF2_Archive_GetEvtWriter
    c_GetEvtWriter.argtypes = [ ctypes.POINTER(Archive), LocationRef ]
    c_GetEvtWriter.restype = ctypes.POINTER(EvtWriter)
    return c_GetEvtWriter(archive, location)

def Archive_GetDefWriter(archive, location):
    c_GetDefWriter = conf.lib.OTF2_Archive_GetDefWriter
    c_GetDefWriter.argtypes = [ ctypes.POINTER(Archive), LocationRef ]
    c_GetDefWriter.restype = ctypes.POINTER(DefWriter)
    return c_GetDefWriter(archive, location)

def Archive_GetGlobalDefWriter(archive):
    c_GetGlobalDefWriter = conf.lib.OTF2_Archive_GetGlobalDefWriter
    c_GetGlobalDefWriter.argtypes = [ ctypes.POINTER(Archive) ]
    c_GetGlobalDefWriter.restype = ctypes.POINTER(GlobalDefWriter)
    return c_GetGlobalDefWriter(archive)

def Archive_GetSnapWriter(archive, location):
    c_GetSnapWriter = conf.lib.OTF2_Archive_GetSnapWriter
    c_GetSnapWriter.argtypes = [ ctypes.POINTER(Archive), LocationRef ]
    c_GetSnapWriter.restype = ctypes.POINTER(SnapWriter)
    return c_GetSnapWriter(archive, location)

def Archive_GetThumbWriter(archive, name, description, type, numberOfSamples, numberOfMetrics, refsToDefs):
    c_GetThumbWriter = conf.lib.OTF2_Archive_GetThumbWriter
    c_GetThumbWriter.argtypes = [ ctypes.POINTER(Archive), StrParam, StrParam, ThumbnailType, ctypes.c_uint32, ctypes.c_uint32, ctypes.POINTER(ctypes.c_uint64) ]
    c_GetThumbWriter.restype = ctypes.POINTER(ThumbWriter)
    return c_GetThumbWriter(archive, ctypes.byref(name), ctypes.byref(description), type, numberOfSamples, numberOfMetrics, ctypes.byref(refsToDefs))

def Archive_GetMarkerWriter(archive):
    c_GetMarkerWriter = conf.lib.OTF2_Archive_GetMarkerWriter
    c_GetMarkerWriter.argtypes = [ ctypes.POINTER(Archive) ]
    c_GetMarkerWriter.restype = ctypes.POINTER(MarkerWriter)
    return c_GetMarkerWriter(archive)

def Archive_GetEvtReader(archive, location):
    c_GetEvtReader = conf.lib.OTF2_Archive_GetEvtReader
    c_GetEvtReader.argtypes = [ ctypes.POINTER(Archive), LocationRef ]
    c_GetEvtReader.restype = ctypes.POINTER(EvtReader)
    return c_GetEvtReader(archive, location)

def Archive_GetGlobalEvtReader(archive):
    c_GetGlobalEvtReader = conf.lib.OTF2_Archive_GetGlobalEvtReader
    c_GetGlobalEvtReader.argtypes = [ ctypes.POINTER(Archive) ]
    c_GetGlobalEvtReader.restype = ctypes.POINTER(GlobalEvtReader)
    return c_GetGlobalEvtReader(archive)

def Archive_GetDefReader(archive, location):
    c_GetDefReader = conf.lib.OTF2_Archive_GetDefReader
    c_GetDefReader.argtypes = [ ctypes.POINTER(Archive), LocationRef ]
    c_GetDefReader.restype = ctypes.POINTER(DefReader)
    return c_GetDefReader(archive, location)

def Archive_GetGlobalDefReader(archive):
    c_GetGlobalDefReader = conf.lib.OTF2_Archive_GetGlobalDefReader
    c_GetGlobalDefReader.argtypes = [ ctypes.POINTER(Archive) ]
    c_GetGlobalDefReader.restype = ctypes.POINTER(GlobalDefReader)
    return c_GetGlobalDefReader(archive)

def Archive_GetSnapReader(archive, location):
    c_GetSnapReader = conf.lib.OTF2_Archive_GetSnapReader
    c_GetSnapReader.argtypes = [ ctypes.POINTER(Archive), LocationRef ]
    c_GetSnapReader.restype = ctypes.POINTER(SnapReader)
    return c_GetSnapReader(archive, location)

def Archive_GetGlobalSnapReader(archive):
    c_GetGlobalSnapReader = conf.lib.OTF2_Archive_GetGlobalSnapReader
    c_GetGlobalSnapReader.argtypes = [ ctypes.POINTER(Archive) ]
    c_GetGlobalSnapReader.restype = ctypes.POINTER(GlobalSnapReader)
    return c_GetGlobalSnapReader(archive)

def Archive_GetThumbReader(archive, number):
    c_GetThumbReader = conf.lib.OTF2_Archive_GetThumbReader
    c_GetThumbReader.argtypes = [ ctypes.POINTER(Archive), ctypes.c_uint32 ]
    c_GetThumbReader.restype = ctypes.POINTER(ThumbReader)
    return c_GetThumbReader(archive, number)

def Archive_GetMarkerReader(archive):
    c_GetMarkerReader = conf.lib.OTF2_Archive_GetMarkerReader
    c_GetMarkerReader.argtypes = [ ctypes.POINTER(Archive) ]
    c_GetMarkerReader.restype = ctypes.POINTER(MarkerReader)
    return c_GetMarkerReader(archive)

def Archive_CloseEvtWriter(archive, writer):
    c_CloseEvtWriter = conf.lib.OTF2_Archive_CloseEvtWriter
    c_CloseEvtWriter.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(EvtWriter) ]
    c_CloseEvtWriter.restype = ErrorCode
    c_CloseEvtWriter.errcheck = HandleErrorCode
    c_CloseEvtWriter(archive, writer)

def Archive_CloseDefWriter(archive, writer):
    c_CloseDefWriter = conf.lib.OTF2_Archive_CloseDefWriter
    c_CloseDefWriter.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(DefWriter) ]
    c_CloseDefWriter.restype = ErrorCode
    c_CloseDefWriter.errcheck = HandleErrorCode
    c_CloseDefWriter(archive, writer)

def Archive_CloseMarkerWriter(archive, writer):
    c_CloseMarkerWriter = conf.lib.OTF2_Archive_CloseMarkerWriter
    c_CloseMarkerWriter.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(MarkerWriter) ]
    c_CloseMarkerWriter.restype = ErrorCode
    c_CloseMarkerWriter.errcheck = HandleErrorCode
    c_CloseMarkerWriter(archive, writer)

def Archive_CloseSnapWriter(archive, writer):
    c_CloseSnapWriter = conf.lib.OTF2_Archive_CloseSnapWriter
    c_CloseSnapWriter.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(SnapWriter) ]
    c_CloseSnapWriter.restype = ErrorCode
    c_CloseSnapWriter.errcheck = HandleErrorCode
    c_CloseSnapWriter(archive, writer)

def Archive_CloseGlobalDefWriter(archive, writer):
    c_CloseGlobalDefWriter = conf.lib.OTF2_Archive_CloseGlobalDefWriter
    c_CloseGlobalDefWriter.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(GlobalDefWriter) ]
    c_CloseGlobalDefWriter.restype = ErrorCode
    c_CloseGlobalDefWriter.errcheck = HandleErrorCode
    c_CloseGlobalDefWriter(archive, writer)

def Archive_CloseEvtReader(archive, reader):
    c_CloseEvtReader = conf.lib.OTF2_Archive_CloseEvtReader
    c_CloseEvtReader.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(EvtReader) ]
    c_CloseEvtReader.restype = ErrorCode
    c_CloseEvtReader.errcheck = HandleErrorCode
    c_CloseEvtReader(archive, reader)

def Archive_CloseThumbReader(archive, reader):
    c_CloseThumbReader = conf.lib.OTF2_Archive_CloseThumbReader
    c_CloseThumbReader.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ThumbReader) ]
    c_CloseThumbReader.restype = ErrorCode
    c_CloseThumbReader.errcheck = HandleErrorCode
    c_CloseThumbReader(archive, reader)

def Archive_CloseGlobalEvtReader(archive, globalEvtReader):
    c_CloseGlobalEvtReader = conf.lib.OTF2_Archive_CloseGlobalEvtReader
    c_CloseGlobalEvtReader.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(GlobalEvtReader) ]
    c_CloseGlobalEvtReader.restype = ErrorCode
    c_CloseGlobalEvtReader.errcheck = HandleErrorCode
    c_CloseGlobalEvtReader(archive, globalEvtReader)

def Archive_CloseDefReader(archive, reader):
    c_CloseDefReader = conf.lib.OTF2_Archive_CloseDefReader
    c_CloseDefReader.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(DefReader) ]
    c_CloseDefReader.restype = ErrorCode
    c_CloseDefReader.errcheck = HandleErrorCode
    c_CloseDefReader(archive, reader)

def Archive_CloseGlobalDefReader(archive, globalDefReader):
    c_CloseGlobalDefReader = conf.lib.OTF2_Archive_CloseGlobalDefReader
    c_CloseGlobalDefReader.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(GlobalDefReader) ]
    c_CloseGlobalDefReader.restype = ErrorCode
    c_CloseGlobalDefReader.errcheck = HandleErrorCode
    c_CloseGlobalDefReader(archive, globalDefReader)

def Archive_CloseSnapReader(archive, reader):
    c_CloseSnapReader = conf.lib.OTF2_Archive_CloseSnapReader
    c_CloseSnapReader.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(SnapReader) ]
    c_CloseSnapReader.restype = ErrorCode
    c_CloseSnapReader.errcheck = HandleErrorCode
    c_CloseSnapReader(archive, reader)

def Archive_CloseGlobalSnapReader(archive, globalSnapReader):
    c_CloseGlobalSnapReader = conf.lib.OTF2_Archive_CloseGlobalSnapReader
    c_CloseGlobalSnapReader.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(GlobalSnapReader) ]
    c_CloseGlobalSnapReader.restype = ErrorCode
    c_CloseGlobalSnapReader.errcheck = HandleErrorCode
    c_CloseGlobalSnapReader(archive, globalSnapReader)

def Archive_CloseMarkerReader(archive, markerReader):
    c_CloseMarkerReader = conf.lib.OTF2_Archive_CloseMarkerReader
    c_CloseMarkerReader.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(MarkerReader) ]
    c_CloseMarkerReader.restype = ErrorCode
    c_CloseMarkerReader.errcheck = HandleErrorCode
    c_CloseMarkerReader(archive, markerReader)

def Archive_GetNumberOfThumbnails(archive):
    c_GetNumberOfThumbnails = conf.lib.OTF2_Archive_GetNumberOfThumbnails
    c_GetNumberOfThumbnails.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_uint32) ]
    c_GetNumberOfThumbnails.restype = ErrorCode
    c_GetNumberOfThumbnails.errcheck = HandleErrorCode
    number = ctypes.c_uint32()
    c_GetNumberOfThumbnails(archive, ctypes.byref(number))
    return number.value

def Archive_SetNumberOfSnapshots(archive, number):
    c_SetNumberOfSnapshots = conf.lib.OTF2_Archive_SetNumberOfSnapshots
    c_SetNumberOfSnapshots.argtypes = [ ctypes.POINTER(Archive), ctypes.c_uint32 ]
    c_SetNumberOfSnapshots.restype = ErrorCode
    c_SetNumberOfSnapshots.errcheck = HandleErrorCode
    c_SetNumberOfSnapshots(archive, number)

def Archive_GetNumberOfSnapshots(archive):
    c_GetNumberOfSnapshots = conf.lib.OTF2_Archive_GetNumberOfSnapshots
    c_GetNumberOfSnapshots.argtypes = [ ctypes.POINTER(Archive), ctypes.POINTER(ctypes.c_uint32) ]
    c_GetNumberOfSnapshots.restype = ErrorCode
    c_GetNumberOfSnapshots.errcheck = HandleErrorCode
    number = ctypes.c_uint32(0)
    c_GetNumberOfSnapshots(archive, ctypes.byref(number))
    return number.value

def Archive_OpenEvtFiles(archive):
    c_OpenEvtFiles = conf.lib.OTF2_Archive_OpenEvtFiles
    c_OpenEvtFiles.argtypes = [ ctypes.POINTER(Archive) ]
    c_OpenEvtFiles.restype = ErrorCode
    c_OpenEvtFiles.errcheck = HandleErrorCode
    c_OpenEvtFiles(archive)

def Archive_CloseEvtFiles(archive):
    c_CloseEvtFiles = conf.lib.OTF2_Archive_CloseEvtFiles
    c_CloseEvtFiles.argtypes = [ ctypes.POINTER(Archive) ]
    c_CloseEvtFiles.restype = ErrorCode
    c_CloseEvtFiles.errcheck = HandleErrorCode
    c_CloseEvtFiles(archive)

def Archive_OpenDefFiles(archive):
    c_OpenDefFiles = conf.lib.OTF2_Archive_OpenDefFiles
    c_OpenDefFiles.argtypes = [ ctypes.POINTER(Archive) ]
    c_OpenDefFiles.restype = ErrorCode
    c_OpenDefFiles.errcheck = HandleErrorCode
    c_OpenDefFiles(archive)

def Archive_CloseDefFiles(archive):
    c_CloseDefFiles = conf.lib.OTF2_Archive_CloseDefFiles
    c_CloseDefFiles.argtypes = [ ctypes.POINTER(Archive) ]
    c_CloseDefFiles.restype = ErrorCode
    c_CloseDefFiles.errcheck = HandleErrorCode
    c_CloseDefFiles(archive)

def Archive_OpenSnapFiles(archive):
    c_OpenSnapFiles = conf.lib.OTF2_Archive_OpenSnapFiles
    c_OpenSnapFiles.argtypes = [ ctypes.POINTER(Archive) ]
    c_OpenSnapFiles.restype = ErrorCode
    c_OpenSnapFiles.errcheck = HandleErrorCode
    c_OpenSnapFiles(archive)

def Archive_CloseSnapFiles(archive):
    c_CloseSnapFiles = conf.lib.OTF2_Archive_CloseSnapFiles
    c_CloseSnapFiles.argtypes = [ ctypes.POINTER(Archive) ]
    c_CloseSnapFiles.restype = ErrorCode
    c_CloseSnapFiles.errcheck = HandleErrorCode
    c_CloseSnapFiles(archive)

def Archive_SelectLocation(archive, location):
    c_SelectLocation = conf.lib.OTF2_Archive_SelectLocation
    c_SelectLocation.argtypes = [ ctypes.POINTER(Archive), LocationRef ]
    c_SelectLocation.restype = ErrorCode
    c_SelectLocation.errcheck = HandleErrorCode
    c_SelectLocation(archive, location)

__all__ = [
    'Archive',
    'Archive_Close',
    'Archive_CloseDefFiles',
    'Archive_CloseDefReader',
    'Archive_CloseDefWriter',
    'Archive_CloseEvtFiles',
    'Archive_CloseEvtReader',
    'Archive_CloseEvtWriter',
    'Archive_CloseGlobalDefReader',
    'Archive_CloseGlobalDefWriter',
    'Archive_CloseGlobalEvtReader',
    'Archive_CloseGlobalSnapReader',
    'Archive_CloseMarkerReader',
    'Archive_CloseMarkerWriter',
    'Archive_CloseSnapFiles',
    'Archive_CloseSnapReader',
    'Archive_CloseSnapWriter',
    'Archive_CloseThumbReader',
    'Archive_GetChunkSize',
    'Archive_GetCompression',
    'Archive_GetCreator',
    'Archive_GetDefReader',
    'Archive_GetDefWriter',
    'Archive_GetDescription',
    'Archive_GetEvtReader',
    'Archive_GetEvtWriter',
    'Archive_GetFileSubstrate',
    'Archive_GetGlobalDefReader',
    'Archive_GetGlobalDefWriter',
    'Archive_GetGlobalEvtReader',
    'Archive_GetGlobalSnapReader',
    'Archive_GetMachineName',
    'Archive_GetMarkerReader',
    'Archive_GetMarkerWriter',
    'Archive_GetNumberOfGlobalDefinitions',
    'Archive_GetNumberOfLocations',
    'Archive_GetNumberOfSnapshots',
    'Archive_GetNumberOfThumbnails',
    'Archive_GetProperty',
    'Archive_GetPropertyNames',
    'Archive_GetSnapReader',
    'Archive_GetSnapWriter',
    'Archive_GetThumbReader',
    'Archive_GetThumbWriter',
    'Archive_GetTraceId',
    'Archive_GetVersion',
    'Archive_IsPrimary',
    'Archive_IsMaster',
    'Archive_Open',
    'Archive_OpenDefFiles',
    'Archive_OpenEvtFiles',
    'Archive_OpenSnapFiles',
    'Archive_SelectLocation',
    'Archive_SetBoolProperty',
    'Archive_SetCollectiveCallbacks',
    'Archive_SetCreator',
    'Archive_SetDescription',
    'Archive_SetFlushCallbacks',
    'Archive_SetHint',
    'Archive_SetLockingCallbacks',
    'Archive_SetMachineName',
    'Archive_SetMemoryCallbacks',
    'Archive_SetNumberOfSnapshots',
    'Archive_SetProperty',
    'Archive_SetSerialCollectiveCallbacks',
    'Archive_SwitchFileMode',
    'Archive_SetDefChunkSize'
]



import _otf2
from .enums import Type
from . import attribute_value


class AttributeList(object):
    def __init__(self, registry, attributes=None):
        if not attributes:
            self._handle = None
            return

        self._handle = _otf2.AttributeList_New()
        for attribute, value in attributes.items():
            self._set_value_by_type(registry, attribute, value)

    def close(self):
        if self._handle is None:
            return
        _otf2.AttributeList_Delete(self._handle)
        self._handle = None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def _set_value_by_type(self, registry, attribute, value):
        _otf2.AttributeList_AddAttribute(self._handle, attribute._ref, attribute.type,
                                         attribute_value.make(registry, value, attribute.type))

    @classmethod
    def _construct(cls, registry, attribute_list):
        if attribute_list is None:
            return None
        count = _otf2.AttributeList_GetNumberOfElements(attribute_list)
        if count == 0:
            return None
        attributes = dict()
        for i in range(count):
            attr_ref, attr_type, attr_value = _otf2.AttributeList_GetAttributeByIndex(
                attribute_list, i)
            attr = registry.attributes[attr_ref]
            assert attr.type == attr_type
            attributes[attr] = attribute_value.parse(registry, attr_value, attr_type)
        return attributes



import ctypes

from .Config import conf, str_errcheck_nullable, EnumBase


class ErrorCode(EnumBase, ctypes.c_int):
    pass

DEPRECATED = ErrorCode(-3)
ABORT = ErrorCode(-2)
WARNING = ErrorCode(-1)
SUCCESS = ErrorCode(0)
ERROR_INVALID = ErrorCode(1)
ERROR_E2BIG = ErrorCode(2)
ERROR_EACCES = ErrorCode(3)
ERROR_EADDRNOTAVAIL = ErrorCode(4)
ERROR_EAFNOSUPPORT = ErrorCode(5)
ERROR_EAGAIN = ErrorCode(6)
ERROR_EALREADY = ErrorCode(7)
ERROR_EBADF = ErrorCode(8)
ERROR_EBADMSG = ErrorCode(9)
ERROR_EBUSY = ErrorCode(10)
ERROR_ECANCELED = ErrorCode(11)
ERROR_ECHILD = ErrorCode(12)
ERROR_ECONNREFUSED = ErrorCode(13)
ERROR_ECONNRESET = ErrorCode(14)
ERROR_EDEADLK = ErrorCode(15)
ERROR_EDESTADDRREQ = ErrorCode(16)
ERROR_EDOM = ErrorCode(17)
ERROR_EDQUOT = ErrorCode(18)
ERROR_EEXIST = ErrorCode(19)
ERROR_EFAULT = ErrorCode(20)
ERROR_EFBIG = ErrorCode(21)
ERROR_EINPROGRESS = ErrorCode(22)
ERROR_EINTR = ErrorCode(23)
ERROR_EINVAL = ErrorCode(24)
ERROR_EIO = ErrorCode(25)
ERROR_EISCONN = ErrorCode(26)
ERROR_EISDIR = ErrorCode(27)
ERROR_ELOOP = ErrorCode(28)
ERROR_EMFILE = ErrorCode(29)
ERROR_EMLINK = ErrorCode(30)
ERROR_EMSGSIZE = ErrorCode(31)
ERROR_EMULTIHOP = ErrorCode(32)
ERROR_ENAMETOOLONG = ErrorCode(33)
ERROR_ENETDOWN = ErrorCode(34)
ERROR_ENETRESET = ErrorCode(35)
ERROR_ENETUNREACH = ErrorCode(36)
ERROR_ENFILE = ErrorCode(37)
ERROR_ENOBUFS = ErrorCode(38)
ERROR_ENODATA = ErrorCode(39)
ERROR_ENODEV = ErrorCode(40)
ERROR_ENOENT = ErrorCode(41)
ERROR_ENOEXEC = ErrorCode(42)
ERROR_ENOLCK = ErrorCode(43)
ERROR_ENOLINK = ErrorCode(44)
ERROR_ENOMEM = ErrorCode(45)
ERROR_ENOMSG = ErrorCode(46)
ERROR_ENOPROTOOPT = ErrorCode(47)
ERROR_ENOSPC = ErrorCode(48)
ERROR_ENOSR = ErrorCode(49)
ERROR_ENOSTR = ErrorCode(50)
ERROR_ENOSYS = ErrorCode(51)
ERROR_ENOTCONN = ErrorCode(52)
ERROR_ENOTDIR = ErrorCode(53)
ERROR_ENOTEMPTY = ErrorCode(54)
ERROR_ENOTSOCK = ErrorCode(55)
ERROR_ENOTSUP = ErrorCode(56)
ERROR_ENOTTY = ErrorCode(57)
ERROR_ENXIO = ErrorCode(58)
ERROR_EOPNOTSUPP = ErrorCode(59)
ERROR_EOVERFLOW = ErrorCode(60)
ERROR_EPERM = ErrorCode(61)
ERROR_EPIPE = ErrorCode(62)
ERROR_EPROTO = ErrorCode(63)
ERROR_EPROTONOSUPPORT = ErrorCode(64)
ERROR_EPROTOTYPE = ErrorCode(65)
ERROR_ERANGE = ErrorCode(66)
ERROR_EROFS = ErrorCode(67)
ERROR_ESPIPE = ErrorCode(68)
ERROR_ESRCH = ErrorCode(69)
ERROR_ESTALE = ErrorCode(70)
ERROR_ETIME = ErrorCode(71)
ERROR_ETIMEDOUT = ErrorCode(72)
ERROR_ETXTBSY = ErrorCode(73)
ERROR_EWOULDBLOCK = ErrorCode(74)
ERROR_EXDEV = ErrorCode(75)
ERROR_END_OF_FUNCTION = ErrorCode(76)
ERROR_INVALID_CALL = ErrorCode(77)
ERROR_INVALID_ARGUMENT = ErrorCode(78)
ERROR_INVALID_RECORD = ErrorCode(79)
ERROR_INVALID_DATA = ErrorCode(80)
ERROR_INVALID_SIZE_GIVEN = ErrorCode(81)
ERROR_UNKNOWN_TYPE = ErrorCode(82)
ERROR_INTEGRITY_FAULT = ErrorCode(83)
ERROR_MEM_FAULT = ErrorCode(84)
ERROR_MEM_ALLOC_FAILED = ErrorCode(85)
ERROR_PROCESSED_WITH_FAULTS = ErrorCode(86)
ERROR_INDEX_OUT_OF_BOUNDS = ErrorCode(87)
ERROR_INVALID_LINENO = ErrorCode(88)
ERROR_END_OF_BUFFER = ErrorCode(89)
ERROR_FILE_INTERACTION = ErrorCode(90)
ERROR_FILE_CAN_NOT_OPEN = ErrorCode(91)
ERROR_INTERRUPTED_BY_CALLBACK = ErrorCode(92)
ERROR_PROPERTY_NAME_INVALID = ErrorCode(93)
ERROR_PROPERTY_EXISTS = ErrorCode(94)
ERROR_PROPERTY_NOT_FOUND = ErrorCode(95)
ERROR_PROPERTY_VALUE_INVALID = ErrorCode(96)
ERROR_FILE_COMPRESSION_NOT_SUPPORTED = ErrorCode(97)
ERROR_DUPLICATE_MAPPING_TABLE = ErrorCode(98)
ERROR_INVALID_FILE_MODE_TRANSITION = ErrorCode(99)
ERROR_COLLECTIVE_CALLBACK = ErrorCode(100)
ERROR_FILE_SUBSTRATE_NOT_SUPPORTED = ErrorCode(101)


def Error_GetName(errorCode):
    c_GetName = conf.lib.OTF2_Error_GetName
    c_GetName.argtypes = [ ErrorCode ]
    c_GetName.restype = ctypes.c_char_p
    c_GetName.errcheck = str_errcheck_nullable
    return c_GetName(errorCode)


def Error_GetDescription(errorCode):
    c_GetDescription = conf.lib.OTF2_Error_GetDescription
    c_GetDescription.argtypes = [ ErrorCode ]
    c_GetDescription.restype = ctypes.c_char_p
    c_GetDescription.errcheck = str_errcheck_nullable
    return c_GetDescription(errorCode)


# This needs ErrorCallback, but ErrorCallback needs variadic arguments which
# cannot be expressed with ctypes ffi.
# def Error_RegisterCallback(errorCallbackIn, userData):
    # c_RegisterCallback = conf.lib.OTF2_Error_RegisterCallback
    # c_RegisterCallback.argtypes = [ ErrorCallback, ctypes.py_object ]
    # c_RegisterCallback.restype = ErrorCallback
    # return c_RegisterCallback(errorCallbackIn, ctypes.py_object(userData))


class Error(Exception):
    '''Represents an generic error that occurred when using OTF2 library

    This is raised in the case where a C library function returns a value
    unlike SUCCESS

    FIXME: Just one generic exception class is not much helpful.
    Get error description from C library.
    '''
    def __init__(self, code):
        self.code = code
        self.name = Error_GetName(code)
        self.description = Error_GetDescription(code)
        self.message = "{} ({}): {}".format(self.name, code.value, self.description)
        Exception.__init__(self, self.message)


def HandleErrorCode(errcode, *args):
    ''' This functions is used for the errcheck attribute of the foreign
        functions. To automatically raise an exception if the ErrorCode
        returned by the C function is not SUCCESS.
    '''
    if errcode != SUCCESS:
        raise Error(errcode)


def errcheck_not_null(value, *_):
    if not value:
        raise Error(ERROR_INVALID)
    return value

# No fun to write all the error codes in here.
# __all__ = ['Error', 'ErrorCode', 'Error_GetDescription', 'HandleErrorCode']




''' Generated by OTF2 Template Engine '''

import ctypes
import traceback
import sys

from .Config import conf, StrParam, str_decode
from .ErrorCodes import ErrorCode, HandleErrorCode
from .GeneralDefinitions import *
from .AttributeValue import AttributeValue
from .Definitions import *
from .Callbacks import callback_wrapper
from .IdMap import IdMap


class DefReaderCallbacks(ctypes.Structure):
    pass

def DefReaderCallbacks_New():
    c_New = conf.lib.OTF2_DefReaderCallbacks_New
    c_New.argtypes = []
    c_New.restype = ctypes.POINTER(DefReaderCallbacks)
    # NOTE: Do not errcheck here. This function returns a value, not an error code
    return c_New()

def DefReaderCallbacks_Delete(defReaderCallbacks):
    c_Delete = conf.lib.OTF2_DefReaderCallbacks_Delete
    c_Delete.argtypes = [ ctypes.POINTER(DefReaderCallbacks) ]
    c_Delete.restype = None
    return c_Delete(defReaderCallbacks)

def DefReaderCallbacks_Clear(defReaderCallbacks):
    c_Clear = conf.lib.OTF2_DefReaderCallbacks_Clear
    c_Clear.argtypes = [ ctypes.POINTER(DefReaderCallbacks) ]
    c_Clear.restype = None
    return c_Clear(defReaderCallbacks)

def _callback_wrapper(type, func, convert_args=None):
    def wrapper(userData, *args):
        if userData:
            py_userData = ctypes.cast(userData, ctypes.py_object).value
        else:
            py_userData = None
        try:
            if convert_args is not None:
                args = convert_args(*args)
            ret = func(py_userData, *args)
            if ret is None:
                ret = CALLBACK_SUCCESS
        except:
            sys.stderr.write("An unhandled python exception has occurred in an "
                             "OTF2_DefReaderCallback:\n")
            sys.stderr.write(traceback.format_exc())
            ret = CALLBACK_ERROR
        return ret.value
    return callback_wrapper(func, wrapper, type)

def _convert_property_union(refered_def, name, type, value):
    union = AttributeValue()
    union._arg_value = value
    return refered_def, name, type, union

_DefReaderCallback_FP_Unknown = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p)

@otf2 for def in defs|local_defs:
_DefReaderCallback_FP_@@def.name@@ = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p@@def.py_funcargtypes(argtypes=false)@@)

@otf2 endfor
def DefReaderCallbacks_SetUnknownCallback(defReaderCallbacks, unknownCallback):
    c_SetUnknownCallback = conf.lib.OTF2_DefReaderCallbacks_SetUnknownCallback
    c_SetUnknownCallback.argtypes = [ ctypes.POINTER(DefReaderCallbacks), _DefReaderCallback_FP_Unknown ]
    c_SetUnknownCallback.restype = ErrorCode
    c_SetUnknownCallback.errcheck = HandleErrorCode
    wrapped_callback = _callback_wrapper(_DefReaderCallback_FP_Unknown,
                                         unknownCallback)
    c_SetUnknownCallback(defReaderCallbacks, wrapped_callback)

@otf2 for def in defs|local_defs:
@otf2  if def == String:
def DefReaderCallbacks_SetStringCallback(defReaderCallbacks, stringCallback):
    c_SetStringCallback = conf.lib.OTF2_DefReaderCallbacks_SetStringCallback
    c_SetStringCallback.argtypes = [ ctypes.POINTER(DefReaderCallbacks), _DefReaderCallback_FP_String ]
    c_SetStringCallback.restype = ErrorCode
    c_SetStringCallback.errcheck = HandleErrorCode
    def convert_string(ref, name):
        return ref, str_decode(name)
    wrapped_callback = _callback_wrapper(_DefReaderCallback_FP_String,
                                         stringCallback, convert_string)
    c_SetStringCallback(defReaderCallbacks, wrapped_callback)

@otf2  elif def == Group:
def DefReaderCallbacks_SetGroupCallback(defReaderCallbacks, groupCallback):
    c_SetGroupCallback = conf.lib.OTF2_DefReaderCallbacks_SetGroupCallback
    c_SetGroupCallback.argtypes = [ ctypes.POINTER(DefReaderCallbacks), _DefReaderCallback_FP_Group ]
    c_SetGroupCallback.restype = ErrorCode
    c_SetGroupCallback.errcheck = HandleErrorCode
    def convert_group(ref, name, group_type, paradigm, group_flags, number_of_metrics,
                      members_array):
        members = [members_array[i] for i in range(number_of_metrics)]
        return ref, name, group_type, paradigm, group_flags, members
    wrapped_callback = _callback_wrapper(_DefReaderCallback_FP_Group,
                                         groupCallback, convert_group)
    c_SetGroupCallback(defReaderCallbacks, wrapped_callback)

@otf2  elif def == MetricClass:
def DefReaderCallbacks_SetMetricClassCallback(defReaderCallbacks, metricClassCallback):
    c_SetMetricClassCallback = conf.lib.OTF2_DefReaderCallbacks_SetMetricClassCallback
    c_SetMetricClassCallback.argtypes = [ ctypes.POINTER(DefReaderCallbacks), _DefReaderCallback_FP_MetricClass ]
    c_SetMetricClassCallback.restype = ErrorCode
    c_SetMetricClassCallback.errcheck = HandleErrorCode
    def convert_metric_class(ref, number_of_metrics, metric_members_array, metric_occurrence,
                             recorder_kind):
        metric_members = [metric_members_array[i] for i in range(number_of_metrics)]
        return ref, metric_members, metric_occurrence, recorder_kind
    wrapped_callback = _callback_wrapper(_DefReaderCallback_FP_MetricClass,
                                         metricClassCallback, convert_metric_class)
    c_SetMetricClassCallback(defReaderCallbacks, wrapped_callback)

@otf2  elif def == CartTopology:
def DefReaderCallbacks_SetCartTopologyCallback(defReaderCallbacks, cartTopologyCallback):
    c_SetCartTopologyCallback = conf.lib.OTF2_DefReaderCallbacks_SetCartTopologyCallback
    c_SetCartTopologyCallback.argtypes = [ ctypes.POINTER(DefReaderCallbacks), _DefReaderCallback_FP_CartTopology ]
    c_SetCartTopologyCallback.restype = ErrorCode
    c_SetCartTopologyCallback.errcheck = HandleErrorCode
    def convert_cart_topology(ref, name, communicator, number_of_dimensions,
                              cart_dimensions_array):
        cart_dimensions = [cart_dimensions_array[i] for i in range(number_of_dimensions)]
        return ref, name, communicator, cart_dimensions
    wrapped_callback = _callback_wrapper(_DefReaderCallback_FP_CartTopology,
                                         cartTopologyCallback, convert_cart_topology)
    c_SetCartTopologyCallback(defReaderCallbacks, wrapped_callback)

@otf2  elif def == CartCoordinate:
def DefReaderCallbacks_SetCartCoordinateCallback(defReaderCallbacks, cartCoordinateCallback):
    c_SetCartCoordinateCallback = conf.lib.OTF2_DefReaderCallbacks_SetCartCoordinateCallback
    c_SetCartCoordinateCallback.argtypes = [ ctypes.POINTER(DefReaderCallbacks), _DefReaderCallback_FP_CartCoordinate ]
    c_SetCartCoordinateCallback.restype = ErrorCode
    c_SetCartCoordinateCallback.errcheck = HandleErrorCode
    def convert_cart_coordinate(cart_topology, rank, number_of_dimensions, coordinates_array):
        coordinates = [coordinates_array[i] for i in range(number_of_dimensions)]
        return cart_topology, rank, coordinates
    wrapped_callback = _callback_wrapper(_DefReaderCallback_FP_CartCoordinate,
                                         cartCoordinateCallback, convert_cart_coordinate)
    c_SetCartCoordinateCallback(defReaderCallbacks, wrapped_callback)

@otf2  else:
def DefReaderCallbacks_Set@@def.name@@Callback(defReaderCallbacks, @@def.lname@@Callback):
    c_Set@@def.name@@Callback = conf.lib.OTF2_DefReaderCallbacks_Set@@def.name@@Callback
    c_Set@@def.name@@Callback.argtypes = [ ctypes.POINTER(DefReaderCallbacks), _DefReaderCallback_FP_@@def.name@@ ]
    c_Set@@def.name@@Callback.restype = ErrorCode
    c_Set@@def.name@@Callback.errcheck = HandleErrorCode
    @otf2 if def.name[-8:] == 'Property' or def.name == 'CallpathParameter':
    convert = _convert_property_union
    @otf2 else:
    convert = None
    @otf2 endif
    wrapped_callback = _callback_wrapper(_DefReaderCallback_FP_@@def.name@@,
                                         @@def.lname@@Callback,
                                         convert)
    c_Set@@def.name@@Callback(defReaderCallbacks, wrapped_callback)

@otf2  endif
@otf2 endfor
__all__ = [
    'DefReaderCallbacks',
    'DefReaderCallbacks_New',
    'DefReaderCallbacks_Delete',
    'DefReaderCallbacks_Clear',
    'DefReaderCallbacks_SetUnknownCallback',
    @otf2 for def in defs|local_defs:
    'DefReaderCallbacks_Set@@def.name@@Callback',
    @otf2 endfor
]



#!/usr/bin/env python

import unittest

from _otf2.Definitions import GROUP_FLAG_GLOBAL_MEMBERS, GROUP_FLAG_NONE
from _otf2.Events import RMA_SYNC_LEVEL_MEMORY, RMA_SYNC_LEVEL_NONE, RMA_SYNC_LEVEL_PROCESS
from _otf2.Definitions import REGION_FLAG_DYNAMIC, REGION_FLAG_NONE, REGION_FLAG_PHASE


class TestSymbolsComplete(unittest.TestCase):
    def test_group_flag(self):
        self.assertEqual(GROUP_FLAG_GLOBAL_MEMBERS | GROUP_FLAG_GLOBAL_MEMBERS,
                         GROUP_FLAG_GLOBAL_MEMBERS)
        self.assertEqual(GROUP_FLAG_GLOBAL_MEMBERS & GROUP_FLAG_GLOBAL_MEMBERS,
                         GROUP_FLAG_GLOBAL_MEMBERS)
        self.assertEqual(GROUP_FLAG_GLOBAL_MEMBERS ^ GROUP_FLAG_GLOBAL_MEMBERS, GROUP_FLAG_NONE)

    def test_rma_sync_level(self):
        self.assertNotEqual(~RMA_SYNC_LEVEL_PROCESS, RMA_SYNC_LEVEL_PROCESS)
        self.assertEqual(RMA_SYNC_LEVEL_MEMORY & RMA_SYNC_LEVEL_NONE, RMA_SYNC_LEVEL_NONE)
        self.assertEqual((RMA_SYNC_LEVEL_MEMORY | RMA_SYNC_LEVEL_NONE | RMA_SYNC_LEVEL_PROCESS) &
                         RMA_SYNC_LEVEL_PROCESS, RMA_SYNC_LEVEL_PROCESS)

    def test_region_flag(self):
        self.assertNotEqual(REGION_FLAG_DYNAMIC| REGION_FLAG_PHASE,  REGION_FLAG_NONE)
        self.assertEqual(REGION_FLAG_DYNAMIC | REGION_FLAG_PHASE,
                         REGION_FLAG_PHASE | REGION_FLAG_DYNAMIC)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import os
import os.path
import unittest
from shutil import rmtree
from tempfile import mkdtemp

import _otf2_combined_writer


class TestOTF2Combined(unittest.TestCase):
    def setUp(self):
        self.tmp_dirname = mkdtemp(prefix=os.path.basename(os.path.abspath(__file__))[:-3] + '_tmp', dir=os.getcwd())

    def tearDown(self):
        if os.getenv('KEEP_TEST_OUTPUT', '') != '':
            print(self.tmp_dirname)
        else:
            rmtree(self.tmp_dirname)

    def test_combined(self):
        _otf2_combined_writer.write_archive(
            os.path.join(self.tmp_dirname, "test_trace"), "traces")


if __name__ == '__main__':
    unittest.main()



import os
import os.path
import unittest
from shutil import rmtree
from tempfile import mkdtemp
import time

import otf2


TIMER_GRANULARITY = 1000000


def t():
    return int(round(time.time() * TIMER_GRANULARITY))


class TestOTF2IO(unittest.TestCase):
    def setUp(self):
        self.old_cwd = os.getcwd()
        self.tmp_dirname = mkdtemp(prefix=os.path.basename(os.path.abspath(__file__))[:-3] + '_tmp', dir=self.old_cwd)

        self.orig_trace = os.path.join(self.tmp_dirname, "orig")
        self.retrace = os.path.join(self.tmp_dirname, "retrc")

        os.mkdir(self.orig_trace)
        os.mkdir(self.retrace)

        os.chdir(self.tmp_dirname)

    def tearDown(self):
        os.chdir(self.old_cwd)
        if os.getenv('KEEP_TEST_OUTPUT', '') != '':
            print(self.tmp_dirname)
        else:
            rmtree(self.tmp_dirname)

    def generate_trace(self, archive_name):
        with otf2.writer.open(archive_name, timer_resolution=TIMER_GRANULARITY) as trace:
            system_tree_node = trace.definitions.system_tree_node("Root", parent=None)
            location_group = trace.definitions.location_group("Process",
                                                              system_tree_parent=system_tree_node)

            writer = trace.event_writer("thread", group=location_group)

            file = trace.definitions.io_regular_file("/home/me/foo.txt", system_tree_node)
            trace.definitions.io_file_property(file, "mount", otf2.Type.STRING, value="/home")
            directory = trace.definitions.io_directory("/etc", system_tree_node)

            posix_paradigm = trace.definitions.io_paradigm("POSIX", "MyFirstPosixParadigm",
                                                           otf2.IoParadigmClass.SERIAL,
                                                           otf2.IoParadigmFlag.NONE,
                                                           {otf2.IoParadigmProperty.VERSION: "1.0"})
            self.assertEqual(posix_paradigm[otf2.IoParadigmProperty.VERSION], "1.0")

            posix_paradigm[otf2.IoParadigmProperty.VERSION] = "2.0"
            self.assertEqual(posix_paradigm[otf2.IoParadigmProperty.VERSION], "2.0")

            posix_paradigm[otf2.IoParadigmProperty.VERSION] =\
                otf2.attribute_value.AttributeValue("3.0", otf2.Type.STRING)
            self.assertEqual(posix_paradigm[otf2.IoParadigmProperty.VERSION], "3.0")

            mpi_io_paradigm = trace.definitions.io_paradigm(
                identification="MPI-IO", name="MyFirstMPI-IOParadigm",
                io_paradigm_class=otf2.IoParadigmClass.PARALLEL,
                io_paradigm_flags=otf2.IoParadigmFlag.NONE,
                properties={otf2.IoParadigmProperty.VERSION:
                            otf2.attribute_value.AttributeValue("0.1", otf2.Type.STRING)})
            self.assertEqual(mpi_io_paradigm[otf2.IoParadigmProperty.VERSION], "0.1")

            stdin = trace.definitions.io_handle('stdin', None, posix_paradigm, otf2.IoHandleFlag.PRE_CREATED)
            trace.definitions.io_pre_created_handle_state(stdin, otf2.IoAccessMode.READ_ONLY)
            stdout = trace.definitions.io_handle('stdout', None, posix_paradigm)
            trace.definitions.io_pre_created_handle_state(stdout, otf2.IoAccessMode.WRITE_ONLY)
            stderr = trace.definitions.io_handle('stderr', None, posix_paradigm)
            trace.definitions.io_pre_created_handle_state(stderr, otf2.IoAccessMode.WRITE_ONLY)

            file_handle = trace.definitions.io_handle('', file, posix_paradigm)
            directory_handle = trace.definitions.io_handle('', directory, mpi_io_paradigm)

            writer.io_create_handle(t(), file_handle, otf2.IoAccessMode.WRITE_ONLY,
                                    otf2.IoCreationFlag.CREATE |
                                    otf2.IoCreationFlag.PATH,
                                    otf2.IoStatusFlag.SYNC |
                                    otf2.IoStatusFlag.CLOSE_ON_EXEC)

    def read_trace(self, archive_name):
        with otf2.reader.open(archive_name) as trace:
            # TODO check something
            pass

    def rewrite_trace(self, old_anchor_path, new_archive_path):
        with otf2.reader.open(old_anchor_path) as trace_reader:
            with otf2.writer.open(new_archive_path,
                                  definitions=trace_reader.definitions) as write_trace:
                for location, event in trace_reader.events:
                    writer = write_trace.event_writer_from_location(location)
                    writer(event)

    def test_rewrite(self):
        self.generate_trace(self.orig_trace)
        self.rewrite_trace(os.path.join(self.orig_trace, "traces.otf2"), self.retrace)
        self.read_trace(os.path.join(self.retrace, "traces.otf2"))


if __name__ == '__main__':
    unittest.main()




''' Generated by OTF2 Template Engine '''

import ctypes
import traceback
import sys

from .Config import conf, StrParam
from .ErrorCodes import ErrorCode, HandleErrorCode
from .GeneralDefinitions import *
from .AttributeList import AttributeList
from .Definitions import *
from .Events import *
from .Callbacks import callback_wrapper


class SnapReaderCallbacks(ctypes.Structure):
    pass

def SnapReaderCallbacks_New():
    c_New = conf.lib.OTF2_SnapReaderCallbacks_New
    c_New.argtypes = []
    c_New.restype = ctypes.POINTER(SnapReaderCallbacks)
    # NOTE: Do not errcheck here. This function returns a value, not an error code
    return c_New()

def SnapReaderCallbacks_Delete(snapReaderCallbacks):
    c_Delete = conf.lib.OTF2_SnapReaderCallbacks_Delete
    c_Delete.argtypes = [ ctypes.POINTER(SnapReaderCallbacks) ]
    c_Delete.restype = None
    return c_Delete(snapReaderCallbacks)

def SnapReaderCallbacks_Clear(snapReaderCallbacks):
    c_Clear = conf.lib.OTF2_SnapReaderCallbacks_Clear
    c_Clear.argtypes = [ ctypes.POINTER(SnapReaderCallbacks) ]
    c_Clear.restype = None
    return c_Clear(snapReaderCallbacks)

def _callback_wrapper(type, func, convert_args=None):
    def wrapper(location, time, userData, attribute_list, *args):
        if userData:
            py_userData = ctypes.cast(userData, ctypes.py_object).value
        else:
            py_userData = None
        try:
            if convert_args is not None:
                args = convert_args(*args)
            if not attribute_list:
                attribute_list = None
            ret = func(location, time, py_userData, attribute_list, *args)
            if ret is None:
                ret = CALLBACK_SUCCESS
        except:
            sys.stderr.write("An unhandled python exception has occurred in an "
                             "OTF2_SnapReaderCallback:\n")
            sys.stderr.write(traceback.format_exc())
            ret = CALLBACK_ERROR
        return ret.value
    return callback_wrapper(func, wrapper, type)

_SnapReaderCallback_FP_Unknown = ctypes.CFUNCTYPE(CallbackCode,
        LocationRef, TimeStamp, ctypes.c_void_p,
        ctypes.POINTER(AttributeList))

@otf2 for snap in snaps:
_SnapReaderCallback_FP_@@snap.name@@ = ctypes.CFUNCTYPE(CallbackCode,
        LocationRef, TimeStamp, ctypes.c_void_p,
        ctypes.POINTER(AttributeList)@@snap.py_funcargtypes(argtypes=False)@@)

@otf2 endfor
def SnapReaderCallbacks_SetUnknownCallback(snapReaderCallbacks, unknownCallbacks):
    c_SetUnknownCallback = conf.lib.OTF2_SnapReaderCallbacks_SetUnknownCallback
    c_SetUnknownCallback.argtypes = [ ctypes.POINTER(SnapReaderCallbacks), _SnapReaderCallback_FP_Unknown ]
    c_SetUnknownCallback.restype = ErrorCode
    c_SetUnknownCallback.errcheck = HandleErrorCode
    wrapped_callback = _callback_wrapper(_SnapReaderCallback_FP_Unknown,
                                         unknownCallback)
    c_SetUnknownCallback(snapReaderCallbacks, wrapped_callback)

@otf2 for snap in snaps:
@otf2  if snap == MetricSnap:
def SnapReaderCallbacks_SetMetricCallback(snapReaderCallbacks, metricCallback):
    c_SetMetricCallback = conf.lib.OTF2_SnapReaderCallbacks_SetMetricCallback
    c_SetMetricCallback.argtypes = [ ctypes.POINTER(SnapReaderCallbacks), _SnapReaderCallback_FP_Metric ]
    c_SetMetricCallback.restype = ErrorCode
    c_SetMetricCallback.errcheck = HandleErrorCode
    def convert_args_metric(metric, number_of_metrics, type_ids_array, metric_values_array):
        type_ids = [type_ids_array[i] for i in range(number_of_metrics)]
        metric_values = [metric_values_array[i] for i in range(number_of_metrics)]
        return metric, type_ids, metric_values
    wrapped_callback = _callback_wrapper(_SnapReaderCallback_FP_Metric,
                                         metricCallback, convert_args_metric)
    c_SetMetricCallback(snapReaderCallbacks, wrapped_callback)

@otf2  else:
def SnapReaderCallbacks_Set@@snap.name@@Callback(snapReaderCallbacks, @@snap.lname@@Callback):
    c_Set@@snap.name@@Callback = conf.lib.OTF2_SnapReaderCallbacks_Set@@snap.name@@
    c_Set@@snap.name@@Callback.argtypes = [ ctypes.POINTER(SnapReaderCallbacks), _SnapReaderCallback_FP_@@snap.name@@ ]
    c_Set@@snap.name@@Callback.restype = ErrorCode
    c_Set@@snap.name@@Callback.errcheck = HandleErrorCode
    wrapped_callback = _callback_wrapper(_SnapReaderCallback_FP_@@snap.lname@@,
                                         @@snap.lname@@Callback)
    c_Set@@snap.name@@Callback(snapReaderCallbacks, wrapped_callback)

@otf2  endif
@otf2 endfor
__all__ = [
    'SnapReaderCallbacks',
    'SnapReaderCallbacks_New',
    'SnapReaderCallbacks_Delete',
    'SnapReaderCallbacks_Clear',
    'SnapReaderCallbacks_SetUnknownCallback',
    @otf2 for snap in snaps:
    'SnapReaderCallbacks_Set@@snap.name@@Callback',
    @otf2 endfor
]




''' Generated by OTF2 Template Engine '''

import ctypes
import traceback
import sys

from .Config import conf, StrParam
from .ErrorCodes import ErrorCode, HandleErrorCode
from .GeneralDefinitions import *
from .AttributeList import AttributeList
from .Definitions import *
from .Events import *
from .Callbacks import callback_wrapper


class EvtReaderCallbacks(ctypes.Structure):
    pass

def EvtReaderCallbacks_New():
    c_New = conf.lib.OTF2_EvtReaderCallbacks_New
    c_New.argtypes = []
    c_New.restype = ctypes.POINTER(EvtReaderCallbacks)
    # NOTE: Do not errcheck here. This function returns a value, not an error code
    return c_New()

def EvtReaderCallbacks_Delete(evtReaderCallbacks):
    c_Delete = conf.lib.OTF2_EvtReaderCallbacks_Delete
    c_Delete.argtypes = [ ctypes.POINTER(EvtReaderCallbacks) ]
    c_Delete.restype = None
    return c_Delete(evtReaderCallbacks)

def EvtReaderCallbacks_Clear(evtReaderCallbacks):
    c_Clear = conf.lib.OTF2_EvtReaderCallbacks_Clear
    c_Clear.argtypes = [ ctypes.POINTER(EvtReaderCallbacks) ]
    c_Clear.restype = None
    return c_Clear(evtReaderCallbacks)

def _callback_wrapper(type, func, convert_args=None):
    def wrapper(location, time, eventPosition, userData, attribute_list, *args):
        if userData:
            py_userData = ctypes.cast(userData, ctypes.py_object).value
        else:
            py_userData = None
        try:
            if convert_args is not None:
                args = convert_args(*args)
            if not attribute_list:
                attribute_list = None
            ret = func(location, time, eventPosition, py_userData, attribute_list, *args)
            if ret is None:
                ret = CALLBACK_SUCCESS
        except:
            sys.stderr.write("An unhandled python exception has occurred in an "
                             "OTF2_EvtReaderCallback:\n")
            sys.stderr.write(traceback.format_exc())
            ret = CALLBACK_ERROR
        return ret.value
    return callback_wrapper(func, wrapper, type)

_EvtReaderCallback_FP_Unknown = ctypes.CFUNCTYPE(CallbackCode,
        LocationRef, TimeStamp, ctypes.c_uint64, ctypes.c_void_p,
        ctypes.POINTER(AttributeList))

@otf2 for event in events:
_EvtReaderCallback_FP_@@event.name@@ = ctypes.CFUNCTYPE(CallbackCode,
        LocationRef, TimeStamp, ctypes.c_uint64, ctypes.c_void_p,
        ctypes.POINTER(AttributeList)@@event.py_funcargtypes(argtypes=False)@@)

@otf2 endfor
def EvtReaderCallbacks_SetUnknownCallback(evtReaderCallbacks, unknownCallback):
    c_SetUnknownCallback = conf.lib.OTF2_EvtReaderCallbacks_SetUnknownCallback
    c_SetUnknownCallback.argtypes = [ ctypes.POINTER(EvtReaderCallbacks), _EvtReaderCallback_FP_Unknown ]
    c_SetUnknownCallback.restype = ErrorCode
    c_SetUnknownCallback.errcheck = HandleErrorCode
    wrapped_callback = _callback_wrapper(_EvtReaderCallback_FP_Unknown,
                                         unknownCallback)
    c_SetUnknownCallback(evtReaderCallbacks, wrapped_callback)

@otf2 for event in events:
@otf2  if event == Metric:
def EvtReaderCallbacks_SetMetricCallback(evtReaderCallbacks, metricCallback):
    c_SetMetricCallback = conf.lib.OTF2_EvtReaderCallbacks_SetMetricCallback
    c_SetMetricCallback.argtypes = [ ctypes.POINTER(EvtReaderCallbacks), _EvtReaderCallback_FP_Metric ]
    c_SetMetricCallback.restype = ErrorCode
    c_SetMetricCallback.errcheck = HandleErrorCode
    def convert_args_metric(metric, number_of_metrics, type_ids_array, metric_values_array):
        type_ids = [type_ids_array[i] for i in range(number_of_metrics)]
        metric_values = [metric_values_array[i] for i in range(number_of_metrics)]
        return metric, type_ids, metric_values
    wrapped_callback = _callback_wrapper(_EvtReaderCallback_FP_Metric,
                                         metricCallback, convert_args_metric)
    c_SetMetricCallback(evtReaderCallbacks, wrapped_callback)

@otf2  elif event == ProgramBegin:
def EvtReaderCallbacks_SetProgramBeginCallback(evtReaderCallbacks, programBeginCallback):
    c_SetProgramBeginCallback = conf.lib.OTF2_EvtReaderCallbacks_SetProgramBeginCallback
    c_SetProgramBeginCallback.argtypes = [ ctypes.POINTER(EvtReaderCallbacks), _EvtReaderCallback_FP_ProgramBegin ]
    c_SetProgramBeginCallback.restype = ErrorCode
    c_SetProgramBeginCallback.errcheck = HandleErrorCode
    def convert_args(program_name, number_of_arguments, arguments_array):
        arguments = [arguments_array[i] for i in range(number_of_arguments)]
        return program_name, arguments
    wrapped_callback = _callback_wrapper(_EvtReaderCallback_FP_ProgramBegin,
                                         programBeginCallback, convert_args)
    c_SetProgramBeginCallback(evtReaderCallbacks, wrapped_callback)

@otf2  else:
def EvtReaderCallbacks_Set@@event.name@@Callback(evtReaderCallbacks, @@event.lname@@Callback):
    c_Set@@event.name@@Callback = conf.lib.OTF2_EvtReaderCallbacks_Set@@event.name@@Callback
    c_Set@@event.name@@Callback.argtypes = [ ctypes.POINTER(EvtReaderCallbacks), _EvtReaderCallback_FP_@@event.name@@ ]
    c_Set@@event.name@@Callback.restype = ErrorCode
    c_Set@@event.name@@Callback.errcheck = HandleErrorCode
    wrapped_callback = _callback_wrapper(_EvtReaderCallback_FP_@@event.name@@,
                                         @@event.lname@@Callback)
    c_Set@@event.name@@Callback(evtReaderCallbacks, wrapped_callback)

@otf2  endif
@otf2 endfor
__all__ = [
    'EvtReaderCallbacks',
    'EvtReaderCallbacks_New',
    'EvtReaderCallbacks_Delete',
    'EvtReaderCallbacks_Clear',
    'EvtReaderCallbacks_SetUnknownCallback',
    @otf2 for event in events:
    'EvtReaderCallbacks_Set@@event.name@@Callback',
    @otf2 endfor
]



import ctypes

from .Config import conf, StrParam
from .ErrorCodes import ErrorCode, HandleErrorCode
from .GlobalEvtReaderCallbacks import GlobalEvtReaderCallbacks


class GlobalEvtReader(ctypes.Structure):
    pass

def GlobalEvtReader_SetCallbacks(reader, callbacks, userData):
    c_SetCallbacks = conf.lib.OTF2_GlobalEvtReader_SetCallbacks
    c_SetCallbacks.argtypes = [ ctypes.POINTER(GlobalEvtReader), ctypes.POINTER(GlobalEvtReaderCallbacks), ctypes.py_object ]
    c_SetCallbacks.restype = ErrorCode
    c_SetCallbacks.errcheck = HandleErrorCode
    c_SetCallbacks(reader, callbacks, ctypes.py_object(userData))

def GlobalEvtReader_ReadEvent(reader):
    c_ReadEvent = conf.lib.OTF2_GlobalEvtReader_ReadEvent
    c_ReadEvent.argtypes = [ ctypes.POINTER(GlobalEvtReader) ]
    c_ReadEvent.restype = ErrorCode
    c_ReadEvent.errcheck = HandleErrorCode
    c_ReadEvent(reader)

def GlobalEvtReader_HasEvent(reader):
    c_HasEvent = conf.lib.OTF2_GlobalEvtReader_HasEvent
    c_HasEvent.argtypes = [ ctypes.POINTER(GlobalEvtReader), ctypes.POINTER(ctypes.c_int) ]
    c_HasEvent.restype = ErrorCode
    c_HasEvent.errcheck = HandleErrorCode
    flag = ctypes.c_int()
    c_HasEvent(reader, ctypes.byref(flag))
    return flag.value

def GlobalEvtReader_ReadEvents(reader, recordsToRead):
    c_ReadEvents = conf.lib.OTF2_GlobalEvtReader_ReadEvents
    c_ReadEvents.argtypes = [ ctypes.POINTER(GlobalEvtReader), ctypes.c_uint64, ctypes.POINTER(ctypes.c_uint64) ]
    c_ReadEvents.restype = ErrorCode
    c_ReadEvents.errcheck = HandleErrorCode
    recordsRead = ctypes.c_uint64()
    c_ReadEvents(reader, recordsToRead, ctypes.byref(recordsRead))
    return recordsRead.value

__all__ = [
    'GlobalEvtReader',
    'GlobalEvtReader_HasEvent',
    'GlobalEvtReader_ReadEvent',
    'GlobalEvtReader_ReadEvents',
    'GlobalEvtReader_SetCallbacks'
]



#!/usr/bin/env python

import os
import os.path
import unittest
from shutil import rmtree
from tempfile import mkdtemp

import _otf2


def pre_flush(user_data, file_type, location, caller_data, final):
    del user_data, file_type, location, caller_data, final
    return _otf2.FLUSH


def post_flush(user_data, file_type, location):
    del user_data, file_type, location
    return get_time()


def get_time():
    get_time.counter += 1
    return get_time.counter


get_time.counter = -1


class TestOTF2AnchorFile(unittest.TestCase):
    def setUp(self):
        self.old_cwd = os.getcwd()
        self.tmp_dirname = mkdtemp(prefix=os.path.basename(os.path.abspath(__file__))[:-3] + '_tmp', dir=self.old_cwd)
        os.chdir(self.tmp_dirname)

    def tearDown(self):
        os.chdir(self.old_cwd)
        if os.getenv('KEEP_TEST_OUTPUT', '') != '':
            print(self.tmp_dirname)
        else:
            rmtree(self.tmp_dirname)

    @staticmethod
    def archive_set_numbers(archive, location_count, global_defs_count):
        assert (location_count * 2 <= global_defs_count)
        glob_def_writer = _otf2.Archive_GetGlobalDefWriter(archive)
        for loc in range(location_count):
            name = "Location #{}".format(loc)
            _otf2.GlobalDefWriter_WriteString(glob_def_writer, loc, name)
            _otf2.GlobalDefWriter_WriteLocation(glob_def_writer, loc, loc,
                                                _otf2.LOCATION_TYPE_CPU_THREAD,
                                                0, loc)
        # now we have writen location_count locations and
        # 2 * location_count definitions
        for fake_string in range(location_count, global_defs_count - location_count):
            _otf2.GlobalDefWriter_WriteString(glob_def_writer, fake_string,
                                              "fake{}".format(fake_string))

        _otf2.Archive_CloseGlobalDefWriter(archive, glob_def_writer)

    def test(self):
        archive_path = "otf2_AnchorFile_trace/trace"
        archive_prefix = "archive"
        archive_name = archive_prefix + ".otf2"

        # Information stored in the trace. */
        chunk_size_events = 1024 * 1024
        chunk_size_defs = 4 * 1024 * 1024
        substrate = _otf2.SUBSTRATE_POSIX
        compression = _otf2.COMPRESSION_NONE
        num_locations = 123
        num_definitions = 456
        machine_name = "TestMachine"
        creator = "TestCreator"
        description = "TestDescription"
        # We will write only two properties. The third one will be deleted,
        # that means it won't be written to anchor file. */
        invalid_property_names = ["",  # empty
                                  "foo",  # no "::" at all
                                  "::foo",  # empty first component 1
                                  "::foo::bar",  # empty first component 2
                                  "foo::::",  # empty middle component 1
                                  "::::foo",  # empty middle component 2
                                  "foo::",  # empty last component 1
                                  "foo::bar::",  # empty last component 2
                                  "foo::ba ::baz",  # invalid character 1
                                  "foo::bar::ba ",  # invalid character 2
                                  # NOTE: None is not an invalid property name but an invalid argument
                                 ]
        string_property_names = ["NamespaceA::Property1", "NamespaceA::Property2", "NamespaceA::Property3"]
        string_property_values = ["Valid", "OK", "true"]
        bool_property_names = ["NamespaceB::BoolProperty1", "NamespaceB::BoolProperty2"]
        bool_property_values = [True, False]
        property_name_to_delete = "Whatever::Property"
        arbitrary_property_value = "Whatever::Value"
        value_to_delete_property = ""

        archive = _otf2.Archive_Open(archive_path,
                                     archive_prefix,
                                     _otf2.FILEMODE_WRITE,
                                     chunk_size_events,
                                     chunk_size_defs,
                                     substrate,
                                     compression)

        flush_callbacks = _otf2.FlushCallbacks(pre_flush=pre_flush, post_flush=post_flush)
        _otf2.Archive_SetFlushCallbacks(archive, flush_callbacks, None)

        # Set machine name, creator and description (optional)
        _otf2.Archive_SetMachineName(archive, machine_name)
        _otf2.Archive_SetCreator(archive, creator)
        _otf2.Archive_SetDescription(archive, description)

        _otf2.Archive_SetSerialCollectiveCallbacks(archive)

        self.archive_set_numbers(archive, num_locations, num_definitions)

        for invalid_property_name in invalid_property_names:
            with self.assertRaises(_otf2.Error) as cm:
                _otf2.Archive_SetProperty(archive, invalid_property_name, "value", True)
            self.assertEqual(cm.exception.code, _otf2.ERROR_PROPERTY_NAME_INVALID,
                             'Incorrect return type for invalid property name: "{}"'.format(invalid_property_name))

        _otf2.Archive_SetProperty(archive,
                                  property_name_to_delete,
                                  value_to_delete_property,
                                  False)

        _otf2.Archive_SetProperty(archive,
                                  property_name_to_delete,
                                  arbitrary_property_value,
                                  False)
        _otf2.Archive_SetProperty(archive,
                                  property_name_to_delete,
                                  value_to_delete_property,
                                  True)

        _otf2.Archive_SetProperty(archive,
                                  string_property_names[0],
                                  arbitrary_property_value,
                                  False)

        _otf2.Archive_SetProperty(archive,
                                  string_property_names[1],
                                  string_property_values[1],
                                  False)

        _otf2.Archive_SetProperty(archive,
                                  string_property_names[0],
                                  string_property_values[0],
                                  True)

        _otf2.Archive_SetProperty(archive,
                                  property_name_to_delete,
                                  arbitrary_property_value,
                                  False)

        _otf2.Archive_SetProperty(archive,
                                  string_property_names[2],
                                  string_property_values[2],
                                  False)

        _otf2.Archive_SetProperty(archive,
                                  property_name_to_delete,
                                  value_to_delete_property,
                                  True)

        _otf2.Archive_SetProperty(archive,
                                  property_name_to_delete,
                                  arbitrary_property_value,
                                  False)
        _otf2.Archive_SetProperty(archive,
                                  property_name_to_delete,
                                  value_to_delete_property,
                                  True)

        _otf2.Archive_SetBoolProperty(archive,
                                      bool_property_names[0],
                                      bool_property_values[0],
                                      False)

        _otf2.Archive_SetBoolProperty(archive,
                                      bool_property_names[1],
                                      bool_property_values[1],
                                      False)

        # Close archive. */
        _otf2.Archive_Close(archive)

        # Open new archive. */
        full_path = os.path.join(archive_path, archive_name)

        reader = _otf2.Reader_Open(full_path)
        _otf2.Reader_SetSerialCollectiveCallbacks(reader)

        version_read = _otf2.Reader_GetVersion(reader)
        version_expected = (_otf2.VERSION_MAJOR, _otf2.VERSION_MINOR, _otf2.VERSION_BUGFIX)
        self.assertEqual(version_read, version_expected, "Version mismatch")

        num_locations_read = _otf2.Reader_GetNumberOfLocations(reader)
        self.assertEqual(num_locations, num_locations_read, "Number of locations does not match")

        num_definitions_read = _otf2.Reader_GetNumberOfGlobalDefinitions(reader, )
        self.assertEqual(num_definitions, num_definitions_read, "Number of definitions does not match")

        (chunk_size_events_read, chunk_size_defs_read) = _otf2.Reader_GetChunkSize(reader)
        self.assertEqual(chunk_size_events, chunk_size_events_read, "Event chunk size does not match")
        self.assertEqual(chunk_size_defs, chunk_size_defs_read, "Definition chunk size does not match")

        substrate_read = _otf2.Reader_GetFileSubstrate(reader)
        self.assertEqual(substrate, substrate_read, "File substrate does not match")

        compression_read = _otf2.Reader_GetCompression(reader)
        self.assertEqual(compression, compression_read, "Compression does not match")

        machine_name_read = _otf2.Reader_GetMachineName(reader)
        self.assertEqual(machine_name, machine_name_read, "Machine name does not match")

        creator_read = _otf2.Reader_GetCreator(reader)
        self.assertEqual(creator, creator_read, "Creator does not match")

        description_read = _otf2.Reader_GetDescription(reader)
        self.assertEqual(description, description_read, "Description does not match")

        # Get additional meta-information (a.k.a. properties) */
        property_names_read = _otf2.Reader_GetPropertyNames(reader)

        self.assertEqual(len(property_names_read), len(string_property_names) + len(bool_property_names),
                         "Number of properties does not match")

        for (property_name_read, property_name_written, property_value_written) \
                in zip(property_names_read, string_property_names + bool_property_names,
                       string_property_values + bool_property_values):
            # First, this test writes #number_of_string_properties properties of type
            # string to the anchor file. Afterwards, #number_of_bool_properties are
            # written to the anchor file.

            # Property names are store in upper case. If everything works correctly,
            #  _otf2.Reader_GetProperty will return our property names converted from
            # mixed case to upper case. Therefore, we also have to convert property
            # names to upper case.
            converted_name = property_name_written.upper()

            self.assertEqual(property_name_read, converted_name, "Property name does not match")

            if isinstance(property_value_written, str):
                property_value_read = _otf2.Reader_GetProperty(reader, property_name_read)
            elif isinstance(property_value_written, bool):
                property_value_read = _otf2.Reader_GetBoolProperty(reader, property_name_read)
            else:
                self.fail("Invalid written property type")
            self.assertEqual(property_value_written, property_value_read, "Property value does not match")

        _otf2.Reader_Close(reader)


if __name__ == '__main__':
    unittest.main()



import ctypes
from functools import wraps

from .GeneralDefinitions import LocationRef, FileType, CallbackCode, Type, FlushType, TimeStamp


_PreFlushCallback_FP = ctypes.CFUNCTYPE(FlushType, ctypes.c_void_p, ctypes.c_uint32,
                                        ctypes.c_uint32, ctypes.c_void_p, ctypes.c_bool)

def callback_wrapper(python_func, wrapper_func, funcptr_type):
    funcptr = funcptr_type(wrapper_func)

    # Attach the wrappers to the original python functions to prevent
    # them from being garbage collected
    try:
        # Works for function / lambdas
        python_func.__wrapper = wrapper_func
        python_func.__wrapper_funcptr = funcptr
    except AttributeError:
        # Probably a bound method, where we cannot set something directly
        setattr(python_func.__self__, '__otf2_wrapper_' + python_func.__name__, wrapper_func)
        setattr(python_func.__self__, '__otf2_wrapper_funcptr_' + python_func.__name__, funcptr)
    return funcptr

def Callbacks_PreFlushCallback(func):
    if func is None:
        return _PreFlushCallback_FP()

    def wrapper(userData, *args, **kwargs):
        if userData:
            py_userData = ctypes.cast(userData, ctypes.py_object).value
        else:
            py_userData = None
        # Why does ctypes want the value (int) here even though FlushType is just a derived int?
        # I think this is a bug.
        return func(py_userData, *args, **kwargs).value

    return callback_wrapper(func, wrapper, _PreFlushCallback_FP)

_PostFlushCallback_FP = ctypes.CFUNCTYPE(TimeStamp, ctypes.c_void_p, ctypes.c_uint32, LocationRef)

def Callbacks_PostFlushCallback(func):
    if func is None:
        return _PostFlushCallback_FP()

    def wrapper(userData, *args, **kwargs):
        if userData:
            py_userData = ctypes.cast(userData, ctypes.py_object).value
        else:
            py_userData = None
        # Note TimeStamp is only an alias, not a derived type, so we do not need .value here
        return func(py_userData, *args, **kwargs)

    return callback_wrapper(func, wrapper, _PostFlushCallback_FP)

class FlushCallbacks(ctypes.Structure):
    def __init__(self, pre_flush, post_flush):
        super(FlushCallbacks, self).__init__(pre_flush=Callbacks_PreFlushCallback(pre_flush),
                                             post_flush=Callbacks_PostFlushCallback(post_flush))

    _fields_ = [
        ("pre_flush", _PreFlushCallback_FP),
        ("post_flush", _PostFlushCallback_FP)
    ]

_MemoryAllocate_FP = ctypes.CFUNCTYPE(ctypes.c_void_p,
                                      FileType,
                                      LocationRef,
                                      ctypes.POINTER(ctypes.c_void_p),
                                      ctypes.c_uint64)

def Callbacks_MemoryAllocate(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _MemoryAllocate_FP(wrapper)

_MemoryFreeAll_FP = ctypes.CFUNCTYPE(ctypes.c_void_p,
                                     FileType,
                                     LocationRef,
                                     ctypes.POINTER(ctypes.c_void_p),
                                     ctypes.c_bool)

def Callbacks_MemoryFreeAll(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _MemoryFreeAll_FP(wrapper)

class MemoryCallbacks(ctypes.Structure):
    _fields_ = [
        ('allocate', _MemoryAllocate_FP),
        ('free_all', _MemoryFreeAll_FP)
    ]

class CollectiveContext(ctypes.Structure):
    pass

# Define Constant
COLLECTIVES_ROOT = ctypes.c_int(0)

_Collectives_GetSize_FP = ctypes.CFUNCTYPE(CallbackCode,
                                           ctypes.c_void_p,
                                           ctypes.POINTER(CollectiveContext),
                                           ctypes.POINTER(ctypes.c_uint32))

def Callbacks_Collectives_GetSize(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _Collectives_GetSize_FP(wrapper)

_Collectives_GetRank_FP = ctypes.CFUNCTYPE(CallbackCode,
                                           ctypes.c_void_p,
                                           ctypes.POINTER(CollectiveContext),
                                           ctypes.POINTER(ctypes.c_uint32))

def Callbacks_Collectives_GetRank(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _Collectives_GetRank_FP(wrapper)

_Collectives_CreateLocalComm_FP = ctypes.CFUNCTYPE(CallbackCode,
                                                   ctypes.c_void_p,
                                                   ctypes.POINTER(ctypes.POINTER(CollectiveContext)),
                                                   ctypes.POINTER(CollectiveContext),
                                                   ctypes.c_uint32,
                                                   ctypes.c_uint32,
                                                   ctypes.c_uint32,
                                                   ctypes.c_uint32,
                                                   ctypes.c_uint32,
                                                   ctypes.c_uint32)

def Callbacks_Collectives_CreateLocalComm(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _Collectives_CreateLocalComm_FP(wrapper)

_Collectives_FreeLocalComm_FP = ctypes.CFUNCTYPE(CallbackCode,
                                                 ctypes.c_void_p,
                                                 ctypes.POINTER(CollectiveContext))

def Callbacks_Collectives_FreeLocalComm(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _Collectives_FreeLocalComm_FP(wrapper)

_Collectives_Barrier_FP = ctypes.CFUNCTYPE(CallbackCode,
                                           ctypes.c_void_p,
                                           ctypes.POINTER(CollectiveContext))

def Callbacks_Collectives_Barrier(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _Collectives_Barrier_FP(wrapper)

_Collectives_Bcast_FP = ctypes.CFUNCTYPE(CallbackCode,
                                         ctypes.c_void_p,
                                         ctypes.POINTER(CollectiveContext),
                                         ctypes.c_void_p,
                                         ctypes.c_uint32,
                                         Type,
                                         ctypes.c_uint32)

def Callbacks_Collectives_Bcast(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _Collectives_Bcast_FP(wrapper)

_Collectives_Gather_FP = ctypes.CFUNCTYPE(CallbackCode,
                                          ctypes.c_void_p,
                                          ctypes.POINTER(CollectiveContext),
                                          ctypes.c_void_p,
                                          ctypes.c_void_p,
                                          ctypes.c_uint32,
                                          Type,
                                          ctypes.c_uint32)

def Callbacks_Collectives_Gather(func):
    def wrapper(userData, commContext, inData, outData,  *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        py_outData = ctypes.cast(outData, ctypes.py_object).value
        return func(py_userData, commContext, inData, py_outData, *args, **kwargs)
    return _Collectives_Gather_FP(wrapper)

_Collectives_Gatherv_FP = ctypes.CFUNCTYPE(CallbackCode,
                                           ctypes.POINTER(CollectiveContext),
                                           ctypes.c_void_p,
                                           ctypes.c_uint32,
                                           ctypes.c_void_p,
                                           ctypes.POINTER(ctypes.c_uint32),
                                           Type,
                                           ctypes.c_uint32)

def Callbacks_Collectives_Gatherv(func):
    def wrapper(userData, commContext, inData, inElements, outData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        py_outData = ctypes.cast(outData, ctypes.py_object).value
        return func(py_userData, commContext, inData, inElements, py_outData, *args, **kwargs)
    return _Collectives_Gatherv_FP(wrapper)

_Collectives_Scatter_FP = ctypes.CFUNCTYPE(CallbackCode,
                                           ctypes.c_void_p,
                                           ctypes.POINTER(CollectiveContext),
                                           ctypes.c_void_p,
                                           ctypes.POINTER(ctypes.c_uint32),
                                           ctypes.c_void_p,
                                           ctypes.c_uint32,
                                           Type,
                                           ctypes.c_uint32)

def Callbacks_Collectives_Scatter(func):
    def wrapper(userData, commContext, inData, outData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        py_outData = ctypes.cast(outData, ctypes.py_object).value
        return func(py_userData, commContext, inData, py_outData, *args, **kwargs)
    return _Collectives_Scatter_FP(wrapper)

_Collectives_Scatterv_FP = ctypes.CFUNCTYPE(CallbackCode,
                                            ctypes.c_void_p,
                                            ctypes.POINTER(CollectiveContext),
                                            ctypes.c_void_p,
                                            ctypes.POINTER(ctypes.c_uint32),
                                            ctypes.c_void_p,
                                            ctypes.c_uint32,
                                            Type,
                                            ctypes.c_uint32)

def Callbacks_Collectives_Scatterv(func):
    def wrapper(userData, commContext, inData, inElements, outData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        py_outData = ctypes.cast(outData, ctypes.py_object).value
        return func(py_userData, commContext, inData, inElements, py_outData, *args, **kwargs)
    return _Collectives_Scatterv_FP(wrapper)

_Collectives_Release_FP = ctypes.CFUNCTYPE(None,
                                           ctypes.c_void_p,
                                           ctypes.POINTER(CollectiveContext),
                                           ctypes.POINTER(CollectiveContext))

def Callbacks_Collectives_Release(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _Collectives_Release_FP(wrapper)

class CollectiveCallbacks(ctypes.Structure):
    _fields_ = [
        ("release", _Collectives_Release_FP),
        ("get_size", _Collectives_GetSize_FP),
        ("get_rank", _Collectives_GetRank_FP),
        ("create_local_comm", _Collectives_CreateLocalComm_FP),
        ("free_local_comm", _Collectives_FreeLocalComm_FP),
        ("barrier", _Collectives_Barrier_FP),
        ("bcast", _Collectives_Bcast_FP),
        ("gather", _Collectives_Gather_FP),
        ("gatherv", _Collectives_Gatherv_FP),
        ("scatter", _Collectives_Scatter_FP),
        ("scatterv", _Collectives_Scatterv_FP)
    ]


class LockObject(ctypes.Structure):
    pass

Lock = ctypes.POINTER(LockObject)

_Locking_Create_FP = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, ctypes.POINTER(Lock))

def Callbacks_Locking_Create(func):
    def wrapper(userData, outData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        py_outData = ctypes.cast(outData, ctypes.py_object).value
        return func(py_userData, py_outData, *args, **kwargs)
    return _Locking_Create_FP(wrapper)

_Locking_Destroy_FP = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, Lock)

def Callbacks_Locking_Destroy(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _Locking_Destroy_FP(wrapper)

_Locking_Lock_FP = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, Lock)

def Callbacks_Locking_Lock(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _Locking_Lock_FP(wrapper)

_Locking_Unlock_FP = ctypes.CFUNCTYPE(CallbackCode, ctypes.c_void_p, Lock)

def Callbacks_Locking_Unlock(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _Locking_Unlock_FP(wrapper)

_Locking_Release_FP = ctypes.CFUNCTYPE(None, ctypes.c_void_p)

def Callbacks_Locking_Release(func):
    def wrapper(userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(py_userData, *args, **kwargs)
    return _Locking_Release_FP(wrapper)

class LockingCallbacks(ctypes.Structure):
    _fields_ = [
        ('release', _Locking_Release_FP),
        ('create', _Locking_Create_FP),
        ('destroy', _Locking_Destroy_FP),
        ('lock', _Locking_Lock_FP),
        ('lock', _Locking_Lock_FP),
        ('unlock', _Locking_Unlock_FP)
    ]

__all__ = [
    'Callbacks_PreFlushCallback',
    'Callbacks_PostFlushCallback',
    'FlushCallbacks',
    'Callbacks_MemoryAllocate',
    'Callbacks_MemoryFreeAll',
    'MemoryCallbacks',
    'CollectiveContext',
    'COLLECTIVES_ROOT',
    'Callbacks_Collectives_GetSize',
    'Callbacks_Collectives_GetRank',
    'Callbacks_Collectives_CreateLocalComm',
    'Callbacks_Collectives_FreeLocalComm',
    'Callbacks_Collectives_Barrier',
    'Callbacks_Collectives_Bcast',
    'Callbacks_Collectives_Gather',
    'Callbacks_Collectives_Gatherv',
    'Callbacks_Collectives_Scatter',
    'Callbacks_Collectives_Scatterv',
    'Callbacks_Collectives_Release',
    'CollectiveCallbacks',
    'LockObject',
    'Lock',
    'Callbacks_Locking_Create',
    'Callbacks_Locking_Destroy',
    'Callbacks_Locking_Lock',
    'Callbacks_Locking_Unlock',
    'Callbacks_Locking_Release',
    'LockingCallbacks'
]



#!/usr/bin/env python

import unittest

import _otf2


class TestOTF2AttributeList(unittest.TestCase):
    def test_1_new_delete(self):
        attr_list = _otf2.AttributeList_New()
        self.assertIsNotNone(attr_list)
        _otf2.AttributeList_Delete(attr_list)

    def test_2_add_get_pop(self):
        attr_list = _otf2.AttributeList_New()
        self.assertIsNotNone(attr_list)
        attr = _otf2.AttributeValue()
        attr.uint32 = 1
        _otf2.AttributeList_AddAttribute(attr_list, 42, _otf2.TYPE_UINT32, attr)

        res = _otf2.AttributeList_TestAttributeByID(attr_list, 42)
        self.assertTrue(res, "test id")

        number_of_elements = _otf2.AttributeList_GetNumberOfElements(attr_list)
        self.assertEqual(number_of_elements, 1, "incorrect number of elements")

        # C-style
        type_ = _otf2.Type()
        id_ = _otf2.AttributeRef()
        _otf2.AttributeList_GetAttributeByID(attr_list, 42, type_, attr)
        self.assertEqual(type_, _otf2.TYPE_UINT32, "incorrect type")
        self.assertNotEqual(type_, _otf2.TYPE_INT32, "incorrect type matches")
        self.assertEqual(attr.uint32, 1, "incorrect value")

        _otf2.AttributeList_GetAttributeByIndex(attr_list, 0, id_, type_, attr)
        self.assertEqual(id_.value, 42, "incorrect id")
        self.assertEqual(type_, _otf2.TYPE_UINT32, "incorrect type")
        self.assertEqual(attr.uint32, 1, "incorrect value")

        # Python-style
        (type_, attr) = _otf2.AttributeList_GetAttributeByID(attr_list, 42)
        self.assertEqual(type_, _otf2.TYPE_UINT32, "incorrect type")
        self.assertNotEqual(type_, _otf2.TYPE_INT32, "incorrect type matches")
        self.assertEqual(attr.uint32, 1, "incorrect value")

        (id_, type_, attr) = _otf2.AttributeList_GetAttributeByIndex(attr_list, 0)
        self.assertEqual(id_, 42, "incorrect id")
        self.assertEqual(type_, _otf2.TYPE_UINT32, "incorrect type")
        self.assertEqual(attr.uint32, 1, "incorrect value")

        (id, type, attr) = _otf2.AttributeList_PopAttribute(attr_list)
        self.assertEqual(id_, 42, "incorrect id")
        self.assertEqual(type_, _otf2.TYPE_UINT32, "incorrect type")
        self.assertEqual(attr.uint32, 1, "incorrect value")

        number_of_elements = _otf2.AttributeList_GetNumberOfElements(attr_list)
        self.assertEqual(number_of_elements, 0, "incorrect number of elements")

        _otf2.AttributeList_Delete(attr_list)

    def test_3_random_remove(self):
        attr_list = _otf2.AttributeList_New()
        self.assertIsNotNone(attr_list)

        ids_to_add = [4, 8, 15, 16, 23, 42]
        # from the end, from the front, from the middle, as index into ids_to_add
        ids_to_remove = [5, 0, 2, 1, 4, 3]

        attr = _otf2.AttributeValue()

        for (i, id_to_add) in enumerate(ids_to_add):
            attr.uint32 = i
            _otf2.AttributeList_AddAttribute(attr_list, id_to_add, _otf2.TYPE_UINT32, attr)

            res = _otf2.AttributeList_TestAttributeByID(attr_list, id_to_add);
            self.assertTrue(res, "failed test id")

            number_of_elements = _otf2.AttributeList_GetNumberOfElements(attr_list)
            self.assertEqual(number_of_elements, i + 1, "incorrect number of elements")

        for (i, id_to_remove) in enumerate(ids_to_remove):
            id_to_add = ids_to_add[id_to_remove]
            (type_, attr) = _otf2.AttributeList_GetAttributeByID(attr_list, id_to_add)
            self.assertEqual(type_, _otf2.TYPE_UINT32)
            self.assertEqual(attr.uint32, id_to_remove)

            _otf2.AttributeList_RemoveAttribute(attr_list, id_to_add)

            ret = _otf2.AttributeList_TestAttributeByID(attr_list, id_to_add)
            self.assertFalse(ret, "found removed test id")

            number_of_elements = _otf2.AttributeList_GetNumberOfElements(attr_list)
            self.assertEqual(number_of_elements, len(ids_to_add) - i - 1, "incorrect number of elements")

        _otf2.AttributeList_Delete(attr_list)


if __name__ == '__main__':
    unittest.main()




''' Generated by OTF2 Template Engine '''

import ctypes

from .Config import EnumBase, EnumBitset
from .GeneralDefinitions import UNDEFINED_UINT32

@otf2 for enum in enums|event_enums:
@otf2  if enum is bitset_enum:
class @@enum.type.py_ctype@@(EnumBitset, @@enum.type.py_underlying_ctype@@):
@otf2  else:
class @@enum.type.py_ctype@@(EnumBase, @@enum.type.py_underlying_ctype@@):
@otf2  endif
    pass

@otf2  for entry in enum.entries|py_def_enum_entries:
@@entry.name[5:]@@ = @@enum.type.py_ctype@@._construct(@@entry.value@@, "@@entry.suffix@@")
@otf2  for alias in entry.aliases:
@@alias.name[5:]@@ = @@enum.type.py_ctype@@._construct(@@entry.value@@, "@@alias.suffix@@")
@otf2  endfor
@otf2  endfor
@otf2  if enum is enum_with_terminator:
@@enum.terminator.name[5:]@@ = @@enum.type.py_ctype@@(@@enum.terminator.value@@)
@otf2  endif

@otf2 endfor
class MetricValue(ctypes.Union):
    _fields_ = [
        ("signed_int", ctypes.c_int64),
        ("unsigned_int", ctypes.c_uint64),
        ("floating_point", ctypes.c_double),
    ]

COLLECTIVE_ROOT_NONE = CollectiveRoot._construct(UNDEFINED_UINT32.value, "NONE")
COLLECTIVE_ROOT_SELF = CollectiveRoot._construct(UNDEFINED_UINT32.value - 1, "SELF")
COLLECTIVE_ROOT_THIS_GROUP = CollectiveRoot._construct(UNDEFINED_UINT32.value - 2, "THIS_GROUP")

__all__ = [
    @otf2 for enum in enums|event_enums:
    '@@enum.type.py_ctype@@',
    @otf2 for entry in enum.entries:
    '@@entry.name[5:]@@',
    @otf2 endfor
    @otf2 if enum is enum_with_terminator:
    '@@enum.terminator.name[5:]@@',
    @otf2 endif
    @otf2 endfor
    'MetricValue'
]



import ctypes

from .Config import conf, EnumBase, errcheck_null_to_none
from .ErrorCodes import ErrorCode, HandleErrorCode


class IdMapMode(EnumBase, ctypes.c_uint8):
    pass

ID_MAP_DENSE = IdMapMode(0)
ID_MAP_SPARSE = IdMapMode(1)

class IdMap(ctypes.Structure):
    pass

_IdMap_TraverseCallback_FP = ctypes.CFUNCTYPE(ctypes.c_void_p,
                                              ctypes.c_uint64,
                                              ctypes.c_uint64,
                                              ctypes.c_void_p)

def IdMap_TraverseCallback(func):
    def wrapper(localId, globalId, userData, *args, **kwargs):
        py_userData = ctypes.cast(userData, ctypes.py_object).value
        return func(localId, globalId, py_userData, *args, **kwargs)
    return _IdMap_TraverseCallback_FP(wrapper)

def IdMap_Create(mode, capacity):
    c_Create = conf.lib.OTF2_IdMap_Create
    c_Create.argtypes = [ IdMapMode, ctypes.c_uint64 ]
    c_Create.restype = ctypes.POINTER(IdMap)
    return c_Create(mode, capacity)

def IdMap_CreateFromUint64Array(mappings, optimizeSize):
    length = len(mappings)
    mappings_array_type = ctypes.c_uint64 * length
    mappings_array = mappings_array_type()
    mappings_array[:] = mappings

    c_CreateFromUint64Array = conf.lib.OTF2_IdMap_CreateFromUint64Array
    c_CreateFromUint64Array.argtypes = [ ctypes.c_uint64, mappings_array_type, ctypes.c_bool ]
    c_CreateFromUint64Array.restype = ctypes.POINTER(IdMap)
    c_CreateFromUint64Array.errcheck = errcheck_null_to_none
    return c_CreateFromUint64Array(length, mappings_array, optimizeSize)

def IdMap_CreateFromUint32Array(length, mappings, optimizeSize):
    length = len(mappings)
    mappings_array_type = ctypes.c_uint32 * length
    mappings_array = mappings_array_type()
    mappings_array[:] = mappings

    c_CreateFromUint32Array = conf.lib.OTF2_IdMap_CreateFromUint32Array
    c_CreateFromUint32Array.argtypes = [ ctypes.c_uint64, mappings_array_type, ctypes.c_bool ]
    c_CreateFromUint32Array.restype = ctypes.POINTER(IdMap)
    c_CreateFromUint32Array.errcheck = errcheck_null_to_none
    return c_CreateFromUint32Array(length, mappings_array, optimizeSize)

def IdMap_Free(instance):
    c_Free = conf.lib.OTF2_IdMap_Free
    c_Free.argtypes = [ ctypes.POINTER(IdMap) ]
    c_Free.restype = None
    return c_Free(instance)

def IdMap_GetSize(instance, size = ctypes.c_uint64()):
    c_GetSize = conf.lib.OTF2_IdMap_GetSize
    c_GetSize.argtypes = [ ctypes.POINTER(IdMap), ctypes.POINTER(ctypes.c_uint64) ]
    c_GetSize.restype = ErrorCode
    c_GetSize.errcheck = HandleErrorCode
    c_GetSize(instance, ctypes.byref(size))
    return size.value

def IdMap_GetMode(instance, mode = IdMapMode()):
    c_GetMode = conf.lib.OTF2_IdMap_GetMode
    c_GetMode.argtypes = [ ctypes.POINTER(IdMap), ctypes.POINTER(IdMapMode) ]
    c_GetMode.restype = ErrorCode
    c_GetMode.errcheck = HandleErrorCode
    c_GetMode(instance, ctypes.byref(mode))
    return mode

def IdMap_Clear(instance):
    c_Clear = conf.lib.OTF2_IdMap_Clear
    c_Clear.argtypes = [ ctypes.POINTER(IdMap) ]
    c_Clear.restype = ErrorCode
    c_Clear.errcheck = HandleErrorCode
    c_Clear(instance)

def IdMap_AddIdPair(instance, localId, globalId):
    c_AddIdPair = conf.lib.OTF2_IdMap_AddIdPair
    c_AddIdPair.argtypes = [ ctypes.POINTER(IdMap), ctypes.c_uint64, ctypes.c_uint64 ]
    c_AddIdPair.restype = ErrorCode
    c_AddIdPair.errcheck = HandleErrorCode
    c_AddIdPair(instance, localId, globalId)

def IdMap_GetGlobalId(instance, localId, globalId = ctypes.c_uint64()):
    c_GetGlobalId = conf.lib.OTF2_IdMap_GetGlobalId
    c_GetGlobalId.argtypes = [ ctypes.POINTER(IdMap), ctypes.c_uint64, ctypes.POINTER(ctypes.c_uint64) ]
    c_GetGlobalId.restype = ErrorCode
    c_GetGlobalId.errcheck = HandleErrorCode
    c_GetGlobalId(instance, localId, ctypes.byref(globalId))
    return globalId.value

def IdMap_GetGlobalIdSave(instance, localId, globalId = ctypes.c_uint64()):
    c_GetGlobalIdSave = conf.lib.OTF2_IdMap_GetGlobalIdSave
    c_GetGlobalIdSave.argtypes = [ ctypes.POINTER(IdMap), ctypes.c_uint64, ctypes.POINTER(ctypes.c_uint64) ]
    c_GetGlobalIdSave.restype = ErrorCode
    c_GetGlobalIdSave.errcheck = HandleErrorCode
    c_GetGlobalIdSave(instance, localId, ctypes.byref(globalId))
    return globalId.value

def IdMap_Traverse(instance, callback, userData):
    c_Traverse = conf.lib.OTF2_IdMap_Traverse
    c_Traverse.argtypes = [ ctypes.POINTER(IdMap), _IdMap_TraverseCallback_FP, ctypes.py_object ]
    c_Traverse.restype = ErrorCode
    c_Traverse.errcheck = HandleErrorCode
    c_Traverse(instance, IdMap_TraverseCallback(callback), ctypes.py_object(userData))

__all__ = [
    'IdMap',
    'IdMap_AddIdPair',
    'IdMap_Clear',
    'IdMap_Create',
    'IdMap_CreateFromUint32Array',
    'IdMap_CreateFromUint64Array',
    'IdMap_Free',
    'IdMap_GetGlobalId',
    'IdMap_GetGlobalIdSave',
    'IdMap_GetMode',
    'IdMap_GetSize',
    'IdMap_Traverse',
    'IdMap_TraverseCallback',
    'IdMapMode',
    'ID_MAP_DENSE',
    'ID_MAP_SPARSE'
]
